//                              -*- Mode: C++ -*-
//: LightYield.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdCareLightYield.hh"
#include "arte/arte.hh"
#include "care/care.hh"
#include "arte/ArteMessage.hh"
#include "care/ESLightYieldCor.hh"

/////////////// The function seen by kuip
#ifdef USEKUIP
extern "C" {

  void eclyc_(){
    CmdCareLightYield light;
    light.readAndExe();
    light.dump();
  }
}
#endif

/////////////// CmdCareLightYield class member function definition

CmdCareLightYield::CmdCareLightYield(){}
CmdCareLightYield::~CmdCareLightYield(){}

void CmdCareLightYield::defValues(){
  cal = 0;
  type = 0;
  degree = 1;
  for(int i=0; i<maxcoeff; i++) coeff[i] = 0.0;
  coeff[0] = 1.0;
}

//--
//   Execute stored command // send data to the relevant object
void CmdCareLightYield::execute(){
  SubCal scal = (SubCal) cal;
  ESLightYieldCor::fill(scal,type,degree, coeff);
}

#ifdef USEKUIP
//--
//   Read acommand from external
void CmdCareLightYield::read(){
  cal = readInt();
  type = readInt();
  degree = readInt();
  for(int i=0; i<maxcoeff; i++) 
    coeff[i] = readFloat();
  if( cal<1 || cal>3 || type<0 || type>3 || degree<0 || degree>10){
    dump();
    arteMsg<<" Range error: cal in [1,3], type in [0,3], degree in [0,10]";
    arteMsg.allp();
    defValues();
  }
}
#endif

void CmdCareLightYield::dump(){
  arteMsg<<"Light Yield correction for calorimeter :"<<cal
	 <<(cal==Inner? " Inner" : (cal==Middle? " Middle ": 
				(cal==Outer? " Outer" : " Wrong! ")));
  arteMsg.allp();
  arteMsg<<" Function type : "<<type<<"  Degree: "<<degree;
  arteMsg.allp();
  for(int i=0; i<degree+1; i++) 
    arteMsg<<" "<<coeff[i];
  arteMsg.allp();
}
