//                              -*- Mode: C++ -*-
//: ESPIDStrategy.C
//
//
// Author: Spighi, Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: spighi@bo.infn.it, villai@bo.infn.it
//
// Created 3/7/97
// Last Update ........
//

#include "care/ESPIDStrategy.hh"
#include "care/EMClusterTree.hh"
#include "care/ECCellID.hh"
#include "arte/ArteMessage.hh"

/*
CLASS
  ESPIDStrategy
   Class for ESPID (pion/electron discrimination) analysis: 
   it evaluates particular quantities to separate pion and electron
   signals using the different lateral spread of the cluster.


*/

//////////////////// ESPIDStrategy Class Definitions ////////////////

ESPIDStrategy::ESPIDStrategy(){
  RPID = -1.;
  likeEle = -1.;
  likeHad = -1.;
}
ESPIDStrategy::~ESPIDStrategy(){}

//--
// Cluster member Analyzer
void ESPIDStrategy::doAnalysis(const EMClusterMember * ){
  arteMsg<<" Error Do Analysis called for ClusterMember within"
      << " a PID Strategy ";
  arteMsg.warn();
}
//--
// TrackInTower analysis
void ESPIDStrategy::doAnalysis(const EMTrackOnEcal *){}
 
//--
// ClusterTree analysis
void ESPIDStrategy::doAnalysis(const EMClusterTree *){}
 
//--
//  Start a cluster analysis
void ESPIDStrategy::beginAnalysis(const EMCluster*, ArtePointer<RCCL> prccl,
				  const EMHitCell* phitc){

  rccl=prccl;
  xcog = rccl->xcog;
  ycog = rccl->ycog;
  hitc = phitc;
  RPID = 0.;
  likeEle = 0.;
  likeHad = 0.;
  typecal = hitc->ncl();
}


//--
// HitTower Analyzer
void ESPIDStrategy::doAnalysis(const EMHitCell * ){}


//--
// Cluster Analyzer
void ESPIDStrategy::doAnalysis(const EMCluster * cluster){

  // loop on clusters
  
  for(EMClusterMember* child = cluster->begin(); 
      child!=0;  child = child->next() ){
    child->analyzeMe(*this);
  }
}


  // Update RCCL bank
void ESPIDStrategy::updatebank(ArtePointer<RCCL> prccl){
  prccl->asym = RPID;
  prccl->lemp = likeEle;
  prccl->lhad = likeHad;
}











