//                              -*- Mode: C++ -*-
//:care.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 29/4/97
// Last Update ........
//
#ifndef CARE_HH_
#define CARE_HH_


/* *
LIBRARY CAlorimeter_REconstruction.
  The HERA-B calorimeter recostruction code is given by the following
  collection of classes. They are built to be used within the ARTE package.


USAGE
  Start with initializing the reconstruction code with a call
  to the careinit_ void function.
  This can be done both in C or C++:

  BEGIN
     careinit_();
  END 

  or in FORTRAN:

  BEGIN
     CALL CAREINIT
  END

  This routines initialize the Look Up Tables (LUTs) used by the program with
  a call to the static function LUTsManager::instance(). If you use 
  the ChronoManager class you need also to initialize it creating an instance
  of the class. 
	At the moment the initialization is not called authomatically
  by arte: it must be called explicitly by the user. A good place to call 
  the initialization routine is the usinit routine in the usinit.C file.
  Then you can call the reconstruction package from ARTE after the 
  digitalization phase. Only one entry point is provided: the void function
  careevnt1_(), that can be called by fortran even before any other 
  routine of reconstruction. This function builds the hierarchical cluster tree
  and does a minimum of cuts and hypotheses on signals. 

  The next step planned but not done already is the interface with ranger to 
  provide starting seeds for track reconstruction and to improve track fitting
  of electrons and positrons in the bending region.

  The final step is the void function careevnt2_() that uses information 
  from ranger, (from TRD and from muon chambers in the future) to label each 
  cluster found on calorimeter: labels could be, for example, electron, MIP, 
  Interacting hadron, gamma, neutral pion or neutral kaon.

  At the end of the analysis job a call to carestop_() is necessary
  to obtain possible information on time performances.


GLOSSARY
  Here a list of used terms and its meaning is reported.

GLOSSARY: ECAL
  The Electromagnetic Calorimeter of the HERA-B experiment, that works around
  the 850 GeV proton beam of the DESY laboratory, Amburg, Germany.

GLOSSARY: MODULE
  A modular element of the ECAL. It has a prism shape with a square section of
  11.2 x 11.2 cm^2. The full ECAL is composed of a grid of 56 x 42 modules of
  4 different types. There are 8 dummy modules (or HOLE modules) that are used 
  to describe that parts of ECAL that are not instrumented due to mechanical
  reasons (space occupied by the beam pipes). There are instrumented modules
  that are composed by instrumented cells: actually there are modules with 1
  cell, with 2x2 cells and with 5x5 cells.

GLOSSARY: CELL
  The minimal element instrumented in ECAL. It is composed by a shashlick
  calorimeter block with a photomultiplier and a digital readout electronics.
  There are 3 different types of cells varying for dimensions, material type 
  and PM type.

GLOSSARY: TOWER
  Synonimous of cell and/or Module. Due to this ambiguity, it is better 
  to avoid it.

GLOSSARY: CLUSTER
  A collection of cells with signals or other clusters (named subclusters).

GLOSSARY: TOPCLUSTER
  A cluster that do not belong to any other exept to the TopClusterList

GLOSSARY: BASECLUSTER
  A cluster that is composed by cells with signals only.

*/

// flags to monitor the performances of the program
#ifndef ONLINE
#  undef PERFORMANCES
#  define USEKUIP
#  define MCARLO
#  define CMATCH
#  undef RTRALINK
#else
#  undef PERFORMANCES
#  undef USEKUIP
#  undef MCARLO
#  undef CMATCH
#  undef RTRALINK
#endif
//--
// Initialization routine for the Calorimeter reconstruction package
extern "C" void careinit_();

//--
/** First step of the calorimeter reconstruction. It builds the 
Hierarchical tree. */	
extern "C" void careevnt1_();

//--
/** Second step of the calorimeter reconstruction.
 It makes connections between clusters and reconstructed tracks. */
  
extern "C" void careevnt2_();

//--

/** Final routine to call to end the job. */
  
extern "C" void carestop_();


#endif // CARE_HH_
