#ifndef LOVER_HH
#define LOVER_HH

#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <math.h>
#include <arte/arte.hh>
#include <arte/RSEG_cmp.hh>
#include "../include/hbook.h"
#include "../include/higz.h"
//stl library
#include <map>
#include <multimap.h>
#include <vector>
#include <set>
#include <algorithm>
#include <list>

using namespace std;

#define NCHAMBERS  10
#define NPLANES   200
#define NGEWA    1500
extern int   npr[NPLANES];
extern float zpl[NPLANES], coeff[NPLANES];

extern int   Nsl_Sec[NCHAMBERS+1][5][5];

extern int   GEDE_cmp[NPLANES],  GEDE_nsl[NPLANES],
  GEDE_sec[NPLANES],  GEDE_lay[NPLANES];
extern float GEDE_sxmn[NPLANES], GEDE_sxmx[NPLANES];
extern float GEDE_symn[NPLANES], GEDE_symx[NPLANES];
extern float GEDE_cos[NPLANES],  GEDE_sin[NPLANES]; 
extern float GEDE_xfw[NPLANES],  GEDE_nwi_x_pit[NPLANES];
extern int   GEDE_gewa[NPLANES];

extern int   base_plane[5];

extern float GEWA_xmin[NGEWA], GEWA_xmax[NGEWA];
extern float GEWA_ymin[NGEWA], GEWA_ymax[NGEWA];
extern float GEWA_zmin[NGEWA], GEWA_zmax[NGEWA];
extern float GEWA_zmean[NGEWA];
extern int   GEWA_gewa[NGEWA];
extern float GEWA_u[NGEWA], GEWA_z[NGEWA], GEWA_ms2[NGEWA];
extern int   ngewa;


extern int acLocalToGlobal(float *, float *, int);
extern int acGlobalToLocal(float *, float *, int);
extern "C"{
  void CatsExtrapolate(ArtePointer<RSEG>, float, 
		       float& , float& , float& , float& , float *);
}

#define ROVERMAXTRACKS 1000

typedef struct{
  int   index;             // internal tracknumber
  int   rtra_rseg;         // flag for rtra (=1) or rseg (=0)
  int   rseg_index;        // 
  int   rtra_index;        // 
  ArtePointer<RSEG> rseg;  // pointer to rseg
  ArtePointer<RTRA> rtra;  // pointer to rtra
  float z;                 // z-coordinate of point on a track
  float x;                 // x-coordinate of point on a track
  float y;                 // y-coordinate of point on a track
  float tx;                // slope dx/dz                     
  float ty;                // slope dy/dz                     
  float p;                 // trackmomentum
  float C[15];             // (triangular) covariance matrix (x,y,tx,ty,p) 
  float w;                 // weight for primary vertexing
  int   used;              // used already for vertexing
  int   nhitb;             // hitb used by the track
} Tracks_t;

typedef map< int, Tracks_t , less<int> >                 Tracks_m;
typedef map< int, Tracks_t , less<int> >::iterator       Tracks_mi;
typedef map< int, Tracks_t , less<int> >::const_iterator Tracks_mci;

typedef map< int,list<int>,less<int> >                     Cluster_m;
typedef map< int,list<int>,less<int> >::iterator           Cluster_mi;


//information about the targets for primary vertices

typedef struct {
                int wnr;
	        int flaghorver;
	        float x;
		float y;
		float z;
		float dx;
		float dy;
                float dz;
                int   n_tr;
                int   n_vx;
} targetstruct ;

typedef map< int, targetstruct, less<int> >           Target_m;
typedef map< int, targetstruct, less<int> >::iterator Target_mi;

typedef multimap<float,Tracks_t,less<float> >  trackmap;
typedef multimap<float,Tracks_t,less<float> >::iterator trackmapit;

typedef map< int, vector<double> , less<int> >         vector_m;

class Lover
{
  private:
          Tracks_m tracks, secondary_tracks;
          RVER vertex;         //vertexpositions  
          Tracks_mi trackincluster[ROVERMAXTRACKS];
          int ntrincluster;
	  int mintracksinvertex;
          int vertex_ok;
          Target_m targets;
          Tracks_mi trackonwire[9][ROVERMAXTRACKS];    // pointers to tracks
          float trackweight[9][ROVERMAXTRACKS];       // track weights on corr. wire
          int ntronwire[9];                 // array size (number of tracks)

          void filltrackmap(Tracks_m&);
          void get_targets(void);
          void trackpropagate(Tracks_t&, float);
          void CatsExtrapolateTrack(Tracks_t& track, float);
          int  target_near(Tracks_t&, int, float);
          void vertex_refit(void);
          void four_dim_inv(float [4][4]);
          void three_dim_inv(float [3][3]);
          void graph_ini(void);
          void graph_close(void);
          void draw_event(void);
          //void verona_init(void);
          void start_hsto(void);
          void fill_ver_hsto(void);
          void fill_track_hsto(void);
          int  getclosestvertex(vector<double>,vector<double>,vector_m,int&);
          int  closestwire(vector<double>,vector<double>,int&, float&, float&, int&);
          float distll(Tracks_t&, Tracks_t&, float&, float&, float&);
 public:
        Lover();
	~Lover();
    	void lover_primaries(void);
        void create_secondary_map(ArtePointer<RTRA>, ArtePointer<RTRA>);
      	void lover_secondary(void);
        //void verona_event(void);
};	 	  
#endif



