 
// *****************************************************************************
//
// source:    DQ_RECO_VZERO2.C
//
// type:      source code
//
// created:   06 Aug 2000
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ_RECO_VZERO2_init()
//            DQ_RECO_VZERO2_evnt( arguments )
//
// hints:     DQ_RECO_VZERO2_init() will be called if DQ/RECO is set
//            DQ_RECO_VZERO2_evnt( arguments ) can be called everywhere
//            use histogram IDs 2000 to 2999
//
// changes:
// 06 Aug 2000 (AG) creation
//
// *****************************************************************************

// standard includes
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"

int DQ_RECO_rcXD0_init();
int DQ_RECO_rcXK892_init();
int DQ_RECO_rcXKs_init();
int DQ_RECO_rcXPhi_init();


//
// function code
//
int
DQ_RECO_VZERO2_init()
{
  ARTE_mark( "DQ_RECO_VZERO2_init 0", 0 );
  cout << "DQ_RECO_VZERO2_init" << endl;

  DQ_RECO_rcXKs_init();     // id=1030 ... 1031
  DQ_RECO_rcXD0_init();     // id=1040 ... 1041
  DQ_RECO_rcXPhi_init();    // id=1050 ... 1051
  DQ_RECO_rcXK892_init();   // id=1060 ... 1061

  ARTE_mark( "DQ_RECO_VZERO2_init 1", 0 );
  return 0;
}

//
// function code
//
int
DQ_RECO_VZERO2_evnt()
{
  ARTE_mark( "DQ_RECO_VZERO2_evnt 0", 0 );
  ARTE_mark( "DQ_RECO_VZERO2_evnt 1", 0 );
  return 0;
}



//
// quality histograms for Ks reco (vds-rich)
//
int DQ_RECO_rcXKs_init() {
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1030 ),"Ks (vds-rich): Event efficiency",5,-0.5,4.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1031 ),"Ks (vds-rich): Mass",30,.44,.56 );
  return 0;
}


//
// quality histograms for D reco
//
int DQ_RECO_rcXD0_init() {
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1040 ),"D: Event efficiency",5,-0.5,4.5 );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1041 ),"D: Mass",30,1.6,2.1 );
  return 0;
}



//
// quality histograms for Phi reco
//
int DQ_RECO_rcXPhi_init() {
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1050 ),"Phi: Event efficiency", 5,-0.5,4.5  );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1051 ),"Phi: Mass",            30, .98,1.11 );
  return 0;
}



//
// quality histograms for K* reco 
//
int DQ_RECO_rcXK892_init() {
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1060 ),"K*: Event efficiency", 5,-0.5,4.5  );
  rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1061 ),"K*: Mass",            30, .5,1.5 );
  return 0;
}



