
// *****************************************************************************
//
// source:    faCheckRhp.C
//
// type:      source code
//
// created:   30 Jun 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    faCheckRhp
//
// called by: anywhere
//
// purpose:   check for rhp dispatch messages if rpm/rhp is initialized
//
// hints:     uses global flag RPMINIT
//
// changes:
// 30 Jun 2000 (AG) creation
// 11 Jul 2000 (AG) use RHPPUB
// 12 Jul 2000 (AG) still use rhp_dispatch
// 13 Jul 2000 (AG) reset histograms after publishing
// 14 Jul 2000 (AG) improved printout
// 27 Jul 2000 (AG) corrected bug
// 07 Aug 2000 (AG) improved printout
// 09 Aug 2000 (AG) introduced print flag
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ONLINE includes
#include "rpm.h"
#include "rhp.h"
#include "grr.h"

// ARTE includes
#include "arte/ArteFlag.h"
#include "farm/farm.hh"

// externals
extern int RPMINIT; // rpm/rhp initialized
extern int RHPPUB;  // use rhp_publish
extern int FADEB;   // debug flag
 
//
// function code
//
int
faCheckRhp( int print )
{
  // rpm/rhp initialized?
  if( !RPMINIT ) return 0;

  // show up
  ARTE_mark( "faCheckRhp 0", 0 );

  // rhp mode
  int rc = GRR_OK;
  
  // always check incomming messages
  while( rc == GRR_OK ){
    ARTE_mark( "rhp_wait_for_one_msg_and_dispatch", 0 );
    rc = rhp_wait_for_one_msg_and_dispatch( NO_BLOCK ); // check for incomming messages
    if( print ) cout << " RHP: rhp_wait_for_one_msg_and_dispatch rc=" << rc << endl;
  }

  // publish histograms
  if( RHPPUB ){

    static int old = -1;
    if( print ) cout << " RHP: "
	 << "nevents=" << arteflag_.nevents << " "
	 << "old="     << old               << endl;
    if( !( arteflag_.nevents%RHPPUB ) && ( arteflag_.nevents != old ) ){
      old = arteflag_.nevents; // store nevents

      // rhp_publish
      ARTE_mark( "rhp_publish", 0 );
      if( ( rc = rhp_publish( 0 ) ) == GRR_OK ){
	if( print ) cout << " RHP: rhp_publish nevt=" << arteflag_.nevents << endl;
	
	// loop over IDs and reset them
	const int  len  = 1000;
	int        max  = 0;
	static int *hid = 0;
	if( !hid ) hid = new int[len];
	rhp_hid( 0, hid, len, &max ); // get all IDs
	if ( print ) cout << " RHP: rhp_hid max=" << max << endl;
	for( int i=0; i<max; i++ ){
	  rc = rhp_reset( hid[i] );           // reset all IDs
	  //cout << "faCheckRhp: " << i << " rhp_reset rc=" << rc << endl;
	}
      }
      // publish error
      else {
	cout << "RHP: ERROR rhp_publish FAILED. rc=" << rc << endl;
	ARTE_error( GRR_SERROR, "rhp_publish FAILED.", rc );
      }
    }
  }
  
  // show up
  ARTE_mark( "faCheckRhp 1", 0 );
  
  return rc;
}
