*CMZ :  1.00/00 27/10/94  18.32.58  by  Dominik Ressing
*CMZ :  0.01/02 13/08/94  21.00.01  by  Dominik Ressing
*CMZ :  0.01/01 24/07/94  13.52.15  by  Dominik Ressing
*-- Author :    Dominik Ressing   21/07/94
************************************************************************
*
* L1DEFPC
* written by Dominik Ressing, 21/07/94 19.51.43
*
* This routines action:
* Reads definiton keys for processor description,
* checks for consistency in the single definitions
* and fills variables in common L1PARAM
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* ierr       update       is set to nonzero on error (else preserved)
*
*
************************************************************************c
      SUBROUTINE L1DEFPC(ierr)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
c
      INTEGER i,j,isup,lsl,l1detsl,iRange,ierr
      EXTERNAL l1detsl
c
c     --- clear arrays with processor definitions first (to catch non filled rows)
      CALL VZERO(L1DEFpredef,L1DEFprelen*L1DEFmaxpre)
      CALL VZERO(L1DEFtfudef,L1DEFtfulen*L1DEFmaxtfu)
      CALL VZERO(L1DEFtpmdef,L1DEFtpmlen*L1DEFmaxtpm)
      CALL VZERO(L1DEFmcudef,L1DEFmculen*L1DEFmaxmcu)
c     --- read the key-file(s)
      CALL l1keyrd
c
c     --- init detector geometry
      CALL L1DETIN(ierr)
c
c     --- loop over all pretriggers
      L1PARnpre = 0
      DO i=1,L1DEFmaxpre
c        --- if inconsistend -> clear row
         IF (L1DEFpredef(L1PREntfu,i).gt.L1DEFpretfu) THEN
          WRITE(L1LUNprn,
     &     '(''PRE'',I3.3,'' has to many TFUs connected -> skipped'')')
     &         i
          CALL VZERO(L1DEFpredef(1,i),L1DEFprelen)
         ELSE IF (L1DEFpredef(L1PREntfu,i).gt.0 ) THEN
            L1PARnpre = L1PARnpre + 1
         END IF
      END DO
      WRITE(L1LUNprn,900) L1PARnpre,'PRE'
c
c
c     --- loop over all TFUs
      L1PARntfu = 0
      L1PARnsup = 0
      DO i=1,L1DEFmaxtfu
         IF ( L1DEFtfudef(L1TFUsup,i).le.0 ) THEN
c          --- this TFU is not used
            CALL VZERO(L1DEFtfudef(1,i),L1DEFtfulen)
         ELSE
           lsl = l1detsl(L1DEFtfudef(L1TFUdec,i))   ! detector superlayer  number
           IF (lsl.le.0) THEN                     ! no valid detector connected
              WRITE(L1LUNprn,
     &           '(''TFU'',I3.3,'' has invalid detector:'',A4)')
     &           i,l1detsl(L1DEFtfudef(L1TFUdec,i))
              CALL VZERO(L1DEFtfudef(1,i),L1DEFtfulen)
           ELSE IF ((L1DEFtfudef(L1TFUnsec,i).lt.1).OR.  ! no or too many sections ?
     &         (L1DEFtfudef(L1TFUnsec,i).gt.L1DEFtfusecs)) THEN
              WRITE(L1LUNprn,
     &       '(''TFU'',I3.3,'' has invalid number of sections:'',I3)')
     &             i,L1DEFtfudef(L1TFUnsec,i)
              CALL VZERO(L1DEFtfudef(1,i),L1DEFtfulen)
           ELSE
              isup = 0
c        --- only valid if at least 1 TFU or a TPM is connected
              DO j=L1TFUtfus,L1DEFtfutfu+L1TFUtfus-1
                 IF ( L1DEFtfudef(j,i).gt.0 ) THEN
                    isup = L1DEFtfudef(L1TFUsup,i)   ! FLT superlayer number
                 END IF
              END DO
              IF (L1DEFtfudef(L1TFUtpm ,i).gt.0) THEN
                 isup = L1DEFtfudef(L1TFUsup,i)      ! FLT superlayer number
              END IF
c
c  --- check connected wire ranges ...
c
              IF ( isup.eq.0 ) THEN
                 WRITE(L1LUNprn,
     &   '(''L1DEFPC ==> TFU'',I3.3,'' has no output connected'')') 
     &              i
                 CALL VZERO(L1DEFtfudef(1,i),L1DEFtfulen)
              END IF
              DO iRange=1,L1DEFtfudef(L1TFUnsec,i)
                 j = L1DEFtfudef(
     &                L1TFUfsec+L1TFUsec+(iRange-1)*L1TFUseclen,i)
                 IF ( j.eq.0 .and. iRange.ne.1 ) THEN
                    isup = 0 ! skipp this TFU
                    WRITE(L1LUNprn,
     &    '(''L1DEFPC ==> TFU'',I3.3,'' invalid range '')')
     &                   i,iRange
                    GO TO 1100
                 END IF
              END DO
 1100         CONTINUE
c
              IF ( isup.gt.0 ) THEN                  ! valid ??
                 L1PARntfu = L1PARntfu + 1
                 L1PARnsup = max(L1PARnsup,isup)
                 L1DEFtfudef(L1TFUdetsl,i) = lsl
              ELSE
                 WRITE(L1LUNprn,
     &  '(''TFU'',I3.3,'' error in definitions -> skipped '')') i
                 CALL VZERO(L1DEFtfudef(1,i),L1DEFtfulen)
              END IF
           END IF
         END IF
      END DO
      WRITE(L1LUNprn,900) L1PARntfu,'TFU'
      WRITE(L1LUNprn,900) L1PARnsup,'SUP'
c
c     --- loop over all TPMs
      L1PARntpm = 0
      DO i=1,L1DEFmaxtpm
c        --- if to many MCUs connected -> clear row
         IF (L1DEFtpmdef(L1TPMnmcu,i).gt.L1DEFtpmmcu) THEN
            WRITE(L1LUNprn,
     & '(''TPM'',I3.3,'' has to many MCUs connected -> skipped'')') i
            CALL VZERO(L1DEFtpmdef(1,i),L1DEFtpmlen)
         ELSE IF (L1DEFtpmdef(L1TPMnmcu,i).gt.0) THEN
            L1PARntpm = L1PARntpm + 1
         END IF
      END DO
      WRITE(L1LUNprn,900) L1PARntpm,'TPM'
c
c     --- loop over all MCUs
      L1PARnmcu = 0
      DO i=1,L1DEFmaxmcu
c        --- if inconsistend -> clear row
         IF (L1DEFmcudef(L1MCUnum,i).ne.i) THEN
            CALL VZERO(L1DEFmcudef(1,i),L1DEFmculen)
         ELSE
            L1PARnmcu = L1PARnmcu + 1
         END IF
      END DO
      WRITE(L1LUNprn,900) L1PARnmcu,'MCU'
c
c
      WRITE(L1LUNprn,100) L1PARdetail
 100  FORMAT(' Simulate hardware at detail level ',i2)
      IF ( L1PARpileup ) THEN
         WRITE(L1LUNprn,*) 'Simulate pile-up of events'
      ELSE
         WRITE(L1LUNprn,*) 'Simulate events one by one'
      END IF
      IF ( L1PARhandev.gt.0 ) THEN
          WRITE(L1LUNprn,*) 'Input events by hand'
      ELSE
          WRITE(L1LUNprn,*) 'Input events from data file'
      END IF
c
c
      RETURN
c
 900  FORMAT(' L1DEFPC ==> found ',i3,1x,a3,'s')
c
      END





