*CMZ :          15/03/96  17.54.24  by  Fedor Ratnikov
*CMZ :  1.01/01 07/01/96  22.13.54  by  Fedor Ratnikov
*CMZ :  1.00/01 04/04/95  23.50.29  by  Fedor Ratnikov
*CMZ :  1.00/00 11/01/95  13.27.51  by  Dominik Ressing
*CMZ :  0.01/03 18/09/94  22.42.04  by  Fedor Ratnikov
*CMZ :  0.01/02 13/09/94  00.34.40  by  Fedor Ratnikov
*-- Author :    Dominik Ressing   25/08/94
************************************************************************
*
* L1TFUMF
* written by Dominik Ressing, 25/08/94 11.25.35
*
* This routines action:
* Returns first crossing-point  in Matrix and flag for matrix kind
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* iDet        in          detector specifier (layer not used)
* iLZ,iLP,iLN in          layers for the 3 projections
* iZ,iP,iN    in          first cells used for zero, positive, negative alpha
* reg         in          region of interest zoomed on detector
* ROI         in          region of intrest as defined in message
* Cross0      out         array of type GeoInf for geometry of first crossing-point
* xLap        out         crossing overlap in x
* yLap        out         crossing overlap in y
* iFlag       out         flag used by l1tfumx for kind of matrix
* iZsh        out         first zero wire to use (relative to iZ-1)
* iPsh        out         first +alpha wire to use (relative to iP-1)
* iNsh        out         first -alpha wire to use (relative to iN-1)
* iHeight     out         height of matrix
*
************************************************************************
c
      SUBROUTINE L1TFUMF(iDet,iLZ,iLP,iLN,iZ,iP,iN,reg,ROI,
     &                   Cross0,xLap,yLap,iFlag,iZsh,iPsh,iNsh,iHeight)
c
      IMPLICIT NONE
c
#include "gean/gcbank.fi"
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
#include "l1si/l1mesdf.fi"
c
      INTEGER iDet(L1DETINFlen),iLZ,iLP,iLN,Ix,
     &        iZ,iP,iN,iFlag,iZsh,iPsh,iNsh,iHeight
      REAL    Cross0(L1GEOINFlen),reg(L1GEOINFlen),ROI(L1ROIlen),
     &        xLap,yLap,xS
c
      REAL    GeoP(L1GEOINFlen),GeoZ(L1GEOINFlen),GeoN(L1GEOINFlen),
     &        rLO,rUp,dx,x0,rNz,zDiff,rDiff,dy,dz,xll,
     &        rMX(L1POINTlen),slope,bot,top,tolY,old,cosal
      INTEGER iLaSave,iLo,iUp,jGeo,jInf,jCell,Nfou,iFir,iLas,iDiff,
     &        iZadd,iPadd,iNadd,ishift
c
      iLaSave = iDet(L1DETSPElayer)
c
c     -- preset defaults
      iZsh = 1
      iPsh = 1
      iNsh = 1
      iFlag = 1
c     -- get detector pointers
      iDet(L1DETSPElayer) = iLZ
      CALL l1detif(iDet,jGeo,jInf,jCell)
c
      IF ( IQ(jInf+L1DETINFwirepad).gt.1 ) THEN ! PAD - like
         iDet(L1DETSPElayer) = iLZ
         CALL l1hitif(iDet,iZ,Cross0)   ! reference cell is the lower left cell
         iFlag = 3
         CALL l1detif(iDet,jGeo,jInf,jCell)
         xLap = Q(jInf+L1DETINFxlap)
         yLap = Q(jInf+L1DETINFylap)
         Cross0(L1GEOINFx0) = Cross0(L1GEOINFx0)+xLap/2.   ! strip
                                                           ! overlaps
         Cross0(L1GEOINFsx) = Cross0(L1GEOINFsx)-xLap
         Cross0(L1GEOINFy0) = Cross0(L1GEOINFy0)+yLap/2.   ! strip
                                                           ! overlaps
         Cross0(L1GEOINFsy) = Cross0(L1GEOINFsy)-yLap
      ELSE                      ! wire-like
c        --- get geometry of left most cells in all 3 projections
c            find offsets of first cells from region boundary:
c               This may be needed since l1celrg limits the range to the detector
C               boundaries!
c
         iDet(L1DETSPElayer) = iLN
         CALL l1detif(iDet,jGeo,jInf,jCell)

         CALL l1hitif(iDet,iN,GeoN)
c
         GeoN(L1GEOINFx0) = GeoN(L1GEOINFx0) +
     &          Q(jInf+L1DETINFxlap)/2.      -       ! strip overlap
     &          (GeoN(L1GEOINFz)-reg(L1GEOINFz))*    ! z-diff to nominal reg
     &            (ROI(L1ROIdxdz)-                   ! track slope in det. system
     &               GeoN(L1GEOINFsin_al)/
     &               GeoN(L1GEOINFcos_al)*ROI(L1ROIdydz))
         GeoN(L1GEOINFsx) = GeoN(L1GEOINFsx) - Q(jInf+L1DETINFxlap)
c
         iDet(L1DETSPElayer) = iLP
         CALL l1hitif(iDet,iP,GeoP)
         CALL l1detif(iDet,jGeo,jInf,jCell)
         cosal = Q(jGeo+L1GEOINFcos_al)
c
         GeoP(L1GEOINFx0) = GeoP(L1GEOINFx0) +
     &          Q(jInf+L1DETINFxlap)/2.      -       ! strip overlap
     &          (GeoP(L1GEOINFz)-reg(L1GEOINFz))*    ! z-diff to nominal reg
     &            (ROI(L1ROIdxdz)-                   ! track slope in det. system
     &               GeoP(L1GEOINFsin_al)/
     &               GeoP(L1GEOINFcos_al)*ROI(L1ROIdydz))
         GeoP(L1GEOINFsx) = GeoP(L1GEOINFsx) - Q(jInf+L1DETINFxlap)
c
         iDet(L1DETSPElayer) = iLZ
         CALL l1hitif(iDet,iZ,GeoZ)
         CALL l1detif(iDet,jGeo,jInf,jCell)
c
         GeoZ(L1GEOINFx0) = GeoZ(L1GEOINFx0) +
     &          Q(jInf+L1DETINFxlap)/2.      -       ! strip overlap
     &          (GeoZ(L1GEOINFz)-reg(L1GEOINFz))*    ! z-diff to nominal reg
     &            ROI(L1ROIdxdz)                     ! track slope in det. system
         GeoZ(L1GEOINFsx) = GeoZ(L1GEOINFsx) - Q(jInf+L1DETINFxlap)
c
c        --- reference crossing point is made by +-alpha projections:
         CALL l1geo2ss(GeoP,GeoN,Cross0)   ! geometry of crossing point
         IF ( Cross0(L1GEOINFx0).lt.-9999. ) THEN ! failed !!
            WRITE(L1LUNprn,*) ' L1TFUMF==> could not calc'//
     &                   ' reference cell!'
            iflag = 0
            RETURN
         END IF
         IF ( L1PARprnlev.ge.3 ) THEN
            WRITE(L1LUNprn,877) Cross0(L1GEOINFx0),Cross0(L1GEOINFsx),
     &           Cross0(L1GEOINFy0),Cross0(L1GEOINFsy)
 877        FORMAT(1x,'L1TFUMF==> Orig. Reference Cell: x0,sx,y0,sy',
     &           4f7.2)
         END IF
c
         xLap = Q(jInf+L1DETINFxlap)/GeoP(L1GEOINFcos_al)
         yLap = Q(jInf+L1DETINFxlap)/GeoP(L1GEOINFsin_al)
c
c        --- move ref-cell cloze to center of ROI
c
         rdiff = 2.*(Cross0(L1GEOINFy0)+0.5*Cross0(L1GEOINFsy)-
     &        (reg(L1GEOINFy0)+0.5*reg(L1GEOINFsy)))/Cross0(L1GEOINFsy)
         idiff = INT(ABS(rdiff)+0.5)
         IF(rdiff.LT.0)    idiff = -idiff
         Cross0(L1GEOINFy0) =
     &        Cross0(L1GEOINFy0)-idiff*0.5*Cross0(L1GEOINFsy)
         Cross0(L1GEOINFx0) =
     &        Cross0(L1GEOINFx0)-ABS(idiff)*0.5*Cross0(L1GEOINFsx)
c
c        --- determine height of matrix in units of reference-cell height
c            distance from top of ROI to top of reference cell:
         rUp = 2.*(reg(L1GEOINFy0)+reg(L1GEOINFsy)-
     &        Cross0(L1GEOINFy0)-Cross0(L1GEOINFsy))/
     &        Cross0(L1GEOINFsy)
         IF ( rUp.gt.0. ) THEN
            iUp = INT(rUp) + 1
         ELSE
            iUp = 0
         END IF
c        --- distance from bottom of ROI to bottom of reference cell:
         rLo = 2.*(Cross0(L1GEOINFy0)-reg(L1GEOINFy0))/
     &        Cross0(L1GEOINFsy)
         IF ( rLo.gt.0. ) THEN
            iLo = INT(rLo) + 1
         ELSE
            iLo = 0
         END IF
         iHeight = 1+(iUp+iLo+1)/2        ! height in matrix units
         bot = Cross0(L1GEOINFy0)-FLOAT(iLo)*Cross0(L1GEOINFsy)/2. ! bot of MX
         top = bot + Cross0(L1GEOINFsy)*FLOAT(iHeight)             ! top of MX
c
c         old = Cross0(L1GEOINFy0)
c         Cross0(L1GEOINFy0) = bot+                 ! reference cell is in middle of MX
c     &         FLOAT(iHeight-1)/2.*Cross0(L1GEOINFsy)
c         rdiff = (Cross0(L1GEOINFy0)-old)/Cross0(L1GEOINFsy)
c         Cross0(L1GEOINFx0) = Cross0(L1GEOINFx0)-
c     &        ABS(rDiff)*Cross0(L1GEOINFsx)
         reg(L1GEOINFx0) = Cross0(L1GEOINFx0) +
     &        0.5*Cross0(L1GEOINFsx)*FLOAT(iHeight)
c
         IF ( L1PARprnlev.ge.2 ) THEN
            WRITE(L1LUNprn,905) reg(L1GEOINFx0), reg(L1GEOINFsx),
     &           bot,top-bot
 905        FORMAT(1x,'L1TFUMF==> Real ROI:       x0,sx,y0,sy',4f7.2)
         END IF
c
c        --- calculate actual height of Matrix and find first P,N wires of this
         rMX(L1POINTdx) = ROI(L1ROIdxdz)
         rMX(L1POINTdy) = ROI(L1ROIdydz)
         rMX(L1POINTx0) = Cross0(L1GEOINFx0)+Cross0(L1GEOINFsx)/2. ! middle of ref
         rMX(L1POINTy0) = Cross0(L1GEOINFy0)+Cross0(L1GEOINFsy)/2. ! middle height
         rMX(L1POINTz0) = reg(L1GEOINFz)
c
         iDet(L1DETSPElayer) = iLN
         CALL l1pohi1(iDet,rMX,iFir)        ! find first hit in -alpha projection
         IF ( iFir.ne.iN ) THEN
            iDiff = iFir-iN
            iNsh  = iNsh + iDiff
            IF ( L1PARprnlev.ge.2 ) THEN
               WRITE(L1LUNprn,900) 'N',iDiff
 900           FORMAT(1x,'L1TFUMF==> Real ROI size shift ',A1,':',I3)
            END IF
         END IF
c
         iDet(L1DETSPElayer) = iLP
         CALL l1pohi1(iDet,rMX,iFir)        ! find first hit in +alpha projection
         IF ( iFir.ne.iP ) THEN
            iDiff = iFir-iP
            iPsh  = iPsh + iDiff
            IF ( L1PARprnlev.ge.2 ) THEN
               WRITE(L1LUNprn,900) 'P',iDiff
            END IF
         END IF
c        --- re-calculate reference crossing point
c         CALL l1geo2s(GeoP,GeoN,Cross0)   ! geometry of crossing point
c
         rNz = float(iHeight)/2.    ! offset of first zero wire
         dx = (GeoZ(L1GEOINFx0)-rNz*GeoZ(L1GEOINFsx) -
     &        Cross0(L1GEOINFx0))/Cross0(L1GEOINFsx)
         IF (dx.gt.0. ) THEN     ! first axial layer inside matrix
            iZsh = iZsh - INT(dx) - 1
         END IF
c
         x0  = GeoZ(L1GEOINFx0)+FLOAT(iZsh-1)*GeoZ(L1GEOINFsx)   ! left of 1st Z cell
         xll = Cross0(L1GEOINFx0)+(rNz-0.5)*Cross0(L1GEOINFsx)   ! lower left crossing
c
         IF(cosal/(1.-cosal).LT.160)   THEN  ! X-diff > 0.2 on full
                                                ! (32 wire) range
            iFlag = -cosal/(1.-cosal)
         ELSEIF ( (x0-xll).lt.xLap/2. ) THEN  ! ll crossing almost covered by 1st Z cell
            iFlag = 2                     ! OR zero wires 1+2 for lower left cross
         ELSE
            iFlag = 1               ! zero wire 1 for lower left cross (1+2 for next)
         END IF
c
c        --- use zero layer for x determination
c
         Cross0(L1GEOINFx0) = GeoZ(L1GEOINFx0)
         Cross0(L1GEOINFsx) = GeoZ(L1GEOINFsx)
         IF ( L1PARprnlev.ge.2 ) THEN
            WRITE(L1LUNprn,910) Cross0(L1GEOINFx0),Cross0(L1GEOINFsx),
     &           Cross0(L1GEOINFy0),Cross0(L1GEOINFsy)
 910        FORMAT(1x,'L1TFUMF==> Reference Cell: x0,sx,y0,sy',4f7.2)
         END IF
c
      END IF
c
      iDet(L1DETSPElayer) = iLaSave
c
      RETURN
c
      END
