
// *****************************************************************************
//
// source:    l4t.C
//
// type:      source code
//
// created:   21 Feb 2000
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY-IfH Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changes:
// 21 Feb 2000 (AG) creation
// 25 Jul 2000 (AG) polished code
// 02 Aug 2000 (AG) commissioned 4LT trigger modes
// 03 Aug 2000 (AG) purged obsolete l4t.hh
// 03 Aug 2000 (AG) SLTHPT -> SLTCUT
// 04 Aug 2000 (AG) cured bug
// 05 Aug 2000 (AG) corrected kuip handling
// 11 Aug 2000 (AG) special selection criteria
// 16 Aug 2000 (AG) introduced SLTYPS
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

// ARTE includes
#include "arte/ArteProto.hh"
#include "farm/farm.hh"
#include "reco/eclass.hh"

// prototypes
extern "C" void kugetc_( char *string, int *len, int len1 );
extern "C" void l4kuip_();

// externals
extern int FADEB;

// globals
int L4TSET = 1;  // L4T (0: 4LT disabled, 1: 4LT enabled)
int L4TSKP = 0;  // skip event
int L4TKUM = -1; // 4LT kumac (-1: not set, 0: OFF, >0: ON)

int L4TMSK = 0;  // class mask to trigger on
int L4TMOD = 0;  // special cases

//
// initialize the Xyz kuip commands
//
static class L4Initer
{
public:
  L4Initer() { atregkuip( l4kuip_, 1 ); } 
} _l4Initer;

//
// program code
//
extern "C"
void
l4_comp_()
{
  ARTE_mark( "l4_comp_ 0", 0 );

  //  
  // read in mode
  //
  int len  = 0;
  int len1 = 6;
  char input[7];
  memset( input, '\0', 7 ); 
  kugetc_( input, &len, len1 );
  input[len] = '\0';
  char *cl  = (char *) malloc( len+1 );
  memset( cl, '\0', len+1 );
  strncpy( cl, input, len );

  // check
  if( !strcmp( cl, "ALL" )    || //  all
      !strcmp( cl, "JPSIEE" ) || //  1
      !strcmp( cl, "JPSIMM" ) || //  2
      !strcmp( cl, "CAL1HT" ) || //  3
      !strcmp( cl, "FCNC"   ) || //  4
      !strcmp( cl, "VZERO"  ) || //  5
      !strcmp( cl, "VZERO2" ) || //  6
      !strcmp( cl, "DVTX"   ) || //  7
      !strcmp( cl, "SLTYPS" ) || //  8
      !strcmp( cl, "SLTCUT" ) || //  9
      !strcmp( cl, "SLTEEB" ) || // 10
      !strcmp( cl, "SLTEE"  ) || // 11
      !strcmp( cl, "SLTMM"  ) || // 12
      !strcmp( cl, "SLTDV"  ) || // 13
      !strcmp( cl, "TLT"    ) || // 14
      !strcmp( cl, "FLTTRG" ) || // 15
      !strcmp( cl, "RNDTRG" )    // 16
      ){
    if( FADEB ) cout << "L4/TRIG class=" << cl << endl;
  }
  // specials ***
  else if( !strcmp( cl, "SKIP"   ) || //  skip event
	   !strcmp( cl, "SLTMMC" )    //  9 & 12
	   ){
    cout << "L4/TRIG SPECIAL class=" << cl << endl;
  }
  // error
  else {
    cout << "L4/TRIG class=" << cl << " UNKOWN!" << endl;
    return;
  }
  
  //
  // read in status
  //
  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = '\0';
  char *status  = (char *) malloc( len+1 );
  sprintf( status  ,"%s", input );
  if( len>0 ) for( int i=0; i<len; i++ ) status[i] = toupper( status[i] );
  if( strcmp( status, "ON"  ) &&
      strcmp( status, "OFF" )    ){
    cout << "L4/TRIG: status=" << status << " UNKOWN!" << endl;
    return;
  }
  
  // set kumac flag
  L4TKUM = 1;

  // ON
  if( !strcmp( status, "ON" ) ){
    
    if( !strcmp( cl, "ALL"    ) ) L4TMSK |= 0xffff;        //  all ON
    if( !strcmp( cl, "JPSIEE" ) ) L4TMSK |= ECLASS_JPSIEE; //  1
    if( !strcmp( cl, "JPSIMM" ) ) L4TMSK |= ECLASS_JPSIMM; //  2
    if( !strcmp( cl, "FCNC"   ) ) L4TMSK |= ECLASS_FCNC;   //  3
    if( !strcmp( cl, "CAL1HT" ) ) L4TMSK |= ECLASS_CAL1HT; //  4
    if( !strcmp( cl, "VZERO"  ) ) L4TMSK |= ECLASS_VZERO;  //  5
    if( !strcmp( cl, "VZERO2" ) ) L4TMSK |= ECLASS_VZERO2; //  6
    if( !strcmp( cl, "DVTX"   ) ) L4TMSK |= ECLASS_DVTX;   //  7
    if( !strcmp( cl, "SLTYPS" ) ) L4TMSK |= ECLASS_SLTYPS; //  8
    if( !strcmp( cl, "SLTCUT" ) ) L4TMSK |= ECLASS_SLTCUT; //  9
    if( !strcmp( cl, "SLTEEB" ) ) L4TMSK |= ECLASS_SLTEEB; // 10
    if( !strcmp( cl, "SLTEE"  ) ) L4TMSK |= ECLASS_SLTEE;  // 11
    if( !strcmp( cl, "SLTMM"  ) ) L4TMSK |= ECLASS_SLTMM;  // 12
    if( !strcmp( cl, "SLTDV"  ) ) L4TMSK |= ECLASS_SLTDV;  // 13
    if( !strcmp( cl, "TLT"    ) ) L4TMSK |= ECLASS_TLT;    // 14
    if( !strcmp( cl, "FLTTRG" ) ) L4TMSK |= ECLASS_FLTTRG; // 15
    if( !strcmp( cl, "RNDTRG" ) ) L4TMSK |= ECLASS_RNDTRG; // 16

    // specials ***
    if( !strcmp( cl, "SKIP"   ) ) L4TMOD = 1; // skip event
    if( !strcmp( cl, "SLTMMC" ) ) L4TMOD = 2; // 9 && 12
  }

  // OFF
  if( !strcmp( status, "OFF" ) ){
    
    if( !strcmp( cl, "ALL"    ) ) L4TMSK &= 0x0;                       //  all OFF
    if( !strcmp( cl, "JPSIEE" ) ) L4TMSK &= ~( int( ECLASS_JPSIEE ) ); //  1
    if( !strcmp( cl, "JPSIMM" ) ) L4TMSK &= ~( int( ECLASS_JPSIMM ) ); //  2
    if( !strcmp( cl, "FCNC"   ) ) L4TMSK &= ~( int( ECLASS_FCNC ) );   //  3
    if( !strcmp( cl, "CAL1HT" ) ) L4TMSK &= ~( int( ECLASS_CAL1HT ) ); //  4
    if( !strcmp( cl, "VZERO"  ) ) L4TMSK &= ~( int( ECLASS_VZERO ) ) ; //  5
    if( !strcmp( cl, "VZERO2" ) ) L4TMSK &= ~( int( ECLASS_VZERO2 ) ); //  6
    if( !strcmp( cl, "DVTX"   ) ) L4TMSK &= ~( int( ECLASS_DVTX ) );   //  7
    if( !strcmp( cl, "SLTYPS" ) ) L4TMSK &= ~( int( ECLASS_SLTYPS ) ); //  8
    if( !strcmp( cl, "SLTCUT" ) ) L4TMSK &= ~( int( ECLASS_SLTCUT ) ); //  9
    if( !strcmp( cl, "SLTEEB" ) ) L4TMSK &= ~( int( ECLASS_SLTEEB ) ); // 10
    if( !strcmp( cl, "SLTEE"  ) ) L4TMSK &= ~( int( ECLASS_SLTEE ) );  // 11
    if( !strcmp( cl, "SLTMM"  ) ) L4TMSK &= ~( int( ECLASS_SLTMM ) );  // 12
    if( !strcmp( cl, "SLTDV"  ) ) L4TMSK &= ~( int( ECLASS_SLTDV ) );  // 13
    if( !strcmp( cl, "TLT"    ) ) L4TMSK &= ~( int( ECLASS_TLT ) );    // 14
    if( !strcmp( cl, "FLTTRG" ) ) L4TMSK &= ~( int( ECLASS_FLTTRG ) ); // 15
    if( !strcmp( cl, "RNDTRG" ) ) L4TMSK &= ~( int( ECLASS_RNDTRG ) ); // 16

    // specials ***
    if( !strcmp( cl, "SKIP"   ) ) L4TMOD = 0; // skip event
    if( !strcmp( cl, "SLTMMC" ) ) L4TMOD = 0; // 9 && 12
  }
  
  // print
  cout << "L4/TRIG: " 
       << cl     << " "
       << status << endl;
  cout << "L4/TRIG: "
       << "mask=0x" << hex << L4TMSK << dec << " "
       << "mode="          << L4TMOD        << " "
       << endl;
  
  ARTE_mark( "l4_comp_ 1", 0 );
}
