//**********************************************************************/
// Created: 23.09.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcMatchConst class
//
//**********************************************************************/
#include "marple/marple.hh"
#include <iostream.h>
#include <iomanip.h>
#include <math.h>

int RcMatchConst::delta_cut_ok( const RcPoint & rp1, 
				const RcPoint & rp2, 
				int iflag  ) const
{
  RcMatchFlag tmpFlag(iflag);
  if( iflag < 0 ) tmpFlag = flag;

  // vertical  angle : does not change
  if( tmpFlag.ifmarked_TY() )
    if( fabs( rp1.ty() - rp2.ty() ) > delta_ty )
      { return 0; }

  // vertical component : the distance between points could be different.
  if( tmpFlag.ifmarked_Y() )
    if( fabs( rp1.z() -rp2.z() ) > 0.1 )
      {
	float y1swim = rp1.y() + (rp2.z()-rp1.z()) * rp1.ty() ;
	if( fabs( y1swim  - rp2.y()  ) > delta_y )
	  { return 0; }
      }else{
	if( fabs( rp1.y()  - rp2.y()  ) > delta_y )
	  { return 0; }
      }

  // horizontal angle : take care of magnetic field
  if(  tmpFlag.ifmarked_TX() || tmpFlag.ifmarked_X() )
    {
      // check that there is magnet field in between
      if( fabs( rp1.z() -rp2.z() ) > 5. && 
	  (fabs(rp1.z()-450)<130 || fabs(rp2.z()-450)<130 ||
	   (rp1.z()-450)*(rp2.z()-450)<0 ) &&
	  fabs( rp1.pe() )>0.006 && // P<150 GeV
	  fabs( rp1.pe() )<2  )     // P>.5 GeV
	{
	  RcPoint tmpPoint = rp1;
	  tmpPoint.prolongate( rp2.z() );
	
	  if( tmpFlag.ifmarked_TX() )
	    if( fabs( tmpPoint.tx() - rp2.tx() ) > delta_tx )
	      { return 0; }
	  
	  if( tmpFlag.ifmarked_X() )
	    if( fabs( tmpPoint.x()  - rp2.x()  ) > delta_x )
	      { return 0; }
	  
	}else{  // no magnet
	  if( tmpFlag.ifmarked_TX() )
	    if( fabs( rp1.tx() - rp2.tx() ) > delta_tx )
	      { return 0; }
	  
	  if( tmpFlag.ifmarked_X() )
	    if( fabs( rp1.z() -rp2.z() ) > 0.1 )
	      {
		float x1swim = rp1.x() + (rp2.z()-rp1.z()) * rp1.tx() ;
		if( fabs( x1swim  - rp2.x()  ) > delta_x )
		  { return 0; }
	      }else{
		if( fabs( rp1.x()  - rp2.x()  ) > delta_x )
		  { return 0; }
	      }
	}
    }

  // momentum
  if( tmpFlag.ifmarked_PE() )
    if( fabs( rp1.pe()  - rp2.pe()  ) > delta_pe )
      { return 0; }
    
  return 1;
}
// energy depended delta cut
int RcMatchConst::energy_delta_cut_ok( const RcPoint & rp1, 
				       const RcClusPoint & rp2, 
				       int iflag  ) const
{
  RcMatchFlag tmpFlag(iflag);
  HepDouble energyCorr = sqrt( fabs(rp2.pe()) ); // 1/sqrt(rp2.e)
  if( energyCorr < MARPLE_INFINITY ) energyCorr = 1;
  
  if( iflag < 0 ) tmpFlag = flag;

  // vertical angle
  if( tmpFlag.ifmarked_TY() )
    if( fabs( rp1.ty() - rp2.ty() ) > delta_ty*energyCorr )
      { return 0; }
  // vertical component : the distance between points could be different.
  if( tmpFlag.ifmarked_Y() )
    if( fabs( rp1.z() -rp2.z() ) > 0.1 )
      {
	float y1swim = rp1.y() + (rp2.z()-rp1.z()) * rp1.ty() ;
	if( fabs( y1swim  - rp2.y()  ) > delta_y*energyCorr )
	  { return 0; }
      }else{
	if( fabs( rp1.y()  - rp2.y()  ) > delta_y*energyCorr )
	  { return 0; }
      }

  // horizontal angle : take care of magnetic field
  if(  tmpFlag.ifmarked_TX() || tmpFlag.ifmarked_X() )
    {
      // check that there is magnet field in between
      if( fabs( rp1.z() -rp2.z() ) > 5. && 
	  (fabs(rp1.z()-450)<130 || fabs(rp2.z()-450)<130 ||
	   (rp1.z()-450)*(rp2.z()-450)<0 ) &&
	  fabs( rp1.pe() )>0.006 && // P<150 GeV
	  fabs( rp1.pe() )<2  )     // P>.5 GeV
	{
	  RcPoint tmpPoint = rp1;
	  tmpPoint.prolongate( rp2.z() );
	
	  if( tmpFlag.ifmarked_TX() )
	    if( fabs( tmpPoint.tx() - rp2.tx() ) > delta_tx*energyCorr )
	      { return 0; }
	  
	  if( tmpFlag.ifmarked_X() )
	    if( fabs( tmpPoint.x()  - rp2.x()  ) > delta_x*energyCorr )
	      { return 0; }
	  
	}else{  // no magnet
	  if( tmpFlag.ifmarked_TX() )
	    if( fabs( rp1.tx() - rp2.tx() ) > delta_tx*energyCorr )
	      { return 0; }
	  
	  if( tmpFlag.ifmarked_X() )
	    if( fabs( rp1.z() -rp2.z() ) > 0.1 )
	      {
		float x1swim = rp1.x() + (rp2.z()-rp1.z()) * rp1.tx() ;
		if( fabs( x1swim  - rp2.x()  ) > delta_x*energyCorr )
		  { return 0; }
	      }else{
		if( fabs( rp1.x()  - rp2.x()  ) > delta_x*energyCorr )
		  { return 0; }
	      }
	}
    }

  // momentum
  if( tmpFlag.ifmarked_PE() )
    if( fabs( rp1.pe()  - rp2.pe()  ) > delta_pe*energyCorr )
      { return 0; }
    
  return 1;
}
//--------------------------------------------------------------

int RcMatchConst::delta_cut_ok( const RcPoint & rp1, 
				const RcLimit & rlimit, 
				int iflag ) const
{
  RcMatchFlag tmpFlag(iflag);
  if( iflag < 0 ) tmpFlag = flag;

  if( tmpFlag.ifmarked_X() )
    {
      if( rp1.x()  < rlimit.x_min  - delta_x  || 
	  rp1.x()  > rlimit.x_max  + delta_x    ) { return 0; }
    }
  
  if( tmpFlag.ifmarked_Y() )
    {
      if( rp1.y()  < rlimit.y_min  - delta_y  || 
	  rp1.y()  > rlimit.y_max  + delta_y    ){ return 0; }
    }

  if( tmpFlag.ifmarked_TX() )
    {
      if( rp1.tx() < rlimit.tx_min - delta_tx || 
	  rp1.tx() > rlimit.tx_max + delta_tx   ){ return 0; }
    }

  if( tmpFlag.ifmarked_TY() )
    {
      if( rp1.ty() < rlimit.ty_min - delta_ty || 
	  rp1.ty() > rlimit.ty_max + delta_ty   ) { return 0; }
    }

  if( tmpFlag.ifmarked_PE() )
    {
      if( rp1.pe() < rlimit.pe_min - delta_pe ||
	  rp1.pe() > rlimit.pe_max + delta_pe   ) { return 0;  }
    }
  return 1;
}
//--------------------------------------------------------------

ostream & operator << (ostream & os, const RcMatchConst &rms)
{
  os << "=========  Constants for Matching  =========" << endl;
  os << " Z coordinate of Matching plane = "<< rms.z_plane << endl;
  os << " Z_window for matching          = [" << rms.z_min << " , " 
     << rms.z_max << "]" << endl;
  if( rms.chi2 < RcMatchConst::maxConsideredChi2 ) 
    os << " the point is accepted if  chi2 < "
       << rms.chi2 << endl;
  else
    os << " there is no cut on chi2 used \n";

  if( rms.use_delta_cut )
    {
      os << " DELTAS_cut :  flag = " << rms.flag << endl ;
      os << "  the following cuts are used :" << endl;

      if( rms.flag.ifmarked_X() )
	os << "  | x1  - x2  | < " << rms.delta_x << endl;
	
      if( rms.flag.ifmarked_Y()  )
	os << "  | y1  - y2  | < " << rms.delta_y << endl;

      if( rms.flag.ifmarked_TX() )
	os << "  | tx1 - tx2 | < " << rms.delta_tx << endl;

      if( rms.flag.ifmarked_TY() )
	os << "  | ty1 - ty2 | < " << rms.delta_ty << endl;
      
      if( rms.flag.ifmarked_PE() )
	os << "  | pe1 - pe2 | < " << rms.delta_pe << endl;
      
    } else {
      os << " no DELTAS_cut is used on {x, y, tx, ty, pe } " << endl;
    }
  if( rms.use_pair_cut )
    {
      os << " PAIR_cut is used : " << endl;
      os << "  " << rms.n_pair_left  
	 << " combination with left object is allowed" << endl;
      os << "  " << rms.n_pair_right 
	 << " combination with right object is allowed" << endl;
    }else{
      os << " no PAIR_cut is used number of combination with objects" << endl;
    }

  return os ;
}

 
