// 
//  Author:  Rainer Mankel                     Created:  23-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// rgtabss
// Fortran interfaced getter function for the seed sets in a gived seed 
// tableau, the pointer of which is given as parameter.
// The seed set pointers are written to a fixed-length array
// which must be defined by the calling program.


#include <math.h>
#include "rang/DomObj.hh"
#include "rang/SeedTableau.hh"
#include "rang/SeedSet.hh"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

extern rangerDomObj* rangerDO;   // global DO pointer

extern "C" {
  void rgtabss_(rangerSeedTableau*& st,
		const int& method,
		const int& maxSeed,
		rangerSeedSet* seedVec[],
		int& nSeed) {

    if (!rangerDO) {
      cout << "rgtabss: rangerDO=" << rangerDO << endl;
      return;
    }

    nSeed = 0;

    // loop over seedSets
    for (vector<rangerSeedSet*>::iterator ssi=st->seedSets().begin();
	 ssi != st->seedSets().end(); ++ssi) {
      if ((*ssi)->method() == method) { 
	if (nSeed < maxSeed) {
	  seedVec[nSeed] = *ssi;
	  ++nSeed;
	}
	else {
	arteMsg << "rgtabss: overflowing nSeed=" << nSeed << endl;
	arteMsg.warn();
	break;
	}
      }
    }
  }
}


    



