*-- Author :    Rainer Mankel   01/10/97

      integer function tfmt2seg(imc,hit_min,zmin,zmax)

*     **** put hits from MC track into a segment for fitting
*     **** return the segment number, or zero 
*     **** The relation between RSEG and RHITs is set.
*     **** The relation between RSEG and HITBs is set.
*     **** The relation between RSEG and MTRA  is set.

*     **** convention: the hits are sorted according to z. The
*     **** "first point" segment parameters refer to the hit
*     **** at lowest z, the "last point" segment parameters refer to 
*     **** the hit at highest z.
*     **** The relations between HITB's and RSEG are established in the 
*     **** order from highest to lowest z. Since the order of retrieval
*     **** in ARTE is reverse, the first HITB retrieved is the hit at
*     **** lowest z, corresponding to the "first point" segment 
*     **** parameters.

*     **** rejects MC particles which curl inside the z range.
*     **** curl condition is determined from inconsistent signs of 
*     **** fmimp_p(..)

      implicit none
#include "arte/Arte.fi"

#include "pris/zmh.fi"

#include "rang/zf.fi"
#include "rang/zfmode.fi"
#include "rang/zfpar.fi"

      integer imc,              ! mtra to be fitted
     +        hit_min           ! minimal number of hits required
      real    zmin,             ! lower z cut for hits
     +        zmax              ! upper z cut for hits

      integer nzfmx_s, nmx
      parameter (nzfmx_s=300, nmx=20)
      integer ind_s(nzfmx_s), hit_s(nzfmx_s), nzf_s, ift, izf,
     +  ii, ih, iso, nso,
     +  mimp_s(nzfmx_s), mimp, iih, hit_array(nmx), nhits, jmc, nnmimp,
     +  cmp, iseg, i, j, rh, i_pack
      integer mimp1,mimp2
      integer message_id_1
      integer nForward, nBackward

      real z_s(nzfmx_s),tfmitoqp
      character*78 c
      logical tfcharg
      logical init

#include "auto/GEO.fi"
#include "auto/HITB.fi"
#include "auto/MTRA.fi"
#include "auto/RSEG.fi"
#include "auto/RHIT.fi"
#include "auto/MIMP.fi"
#include "auto/TOPP.fi"

#include "pris/zmpack.fi"
      data message_id_1 /1/, init /.false./

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/HITB.ff"
#include "auto/MTRA.ff"
#include "auto/RSEG.ff"
#include "auto/RHIT.ff"
#include "auto/MIMP.ff"
#include "auto/TOPP.ff"


      tfmt2seg = 0              ! return value defaults to zero

      ift = 1

*     **** get izf of hits, sorted after decreasing z
      nzf_s = 0

*     **** initialize /zmh/ if needed
      call tusomh()

      if (.not.tfcharg(imc)) go to 999

*     **** loop over hits in sorted order
      if (imc.gt.nmc_mx) then
         print *,' () tfmt2seg: too large imc=', imc
         return
      endif
      nso = nmimmh(imc)

      nForward = 0
      nBackward = 0

      do 30 iso=nso,1,-1
         mimp = imimmh( istamh(imc) + iso )
*        **** get hits associated with mimp
         call tumimph(mimp,nmx,hit_array,nhits)

**         if (igede_cmp(imimp_gede(mimp)).eq.1.or.
**     +        igede_cmp(imimp_gede(mimp)).eq.3) then
**            if (nhits.gt.1) go to 30 ! only 1 hit per mimp in strip dev
**         endif

         if (fmimp_z(mimp).gt.zmin.and.fmimp_z(mimp).lt.zmax) then
            if (fmimp_p(mimp).gt.0.) then
               nForward = nForward + 1
            elseif (fmimp_p(mimp).lt.0.) then
               nBackward = nBackward + 1
            endif
         endif

*        **** loop over associated hits
         do 40 iih=1,nhits
            ih = hit_array(iih)
            if (ih.le.0.or.ih.gt.iq(jhitb+2)) then
               write(c,'((a),i8,(a),2i10)') 
     +              ' () tfmt2seg: invalid ih=',ih,
     +              ' at iih,mimp=',iih,mimp
               call mewarn(c)
               go to 999
            endif

*           **** no merged points
            call tuhmimp(ih,jmc,mimp,nnmimp)
            if (nnmimp.gt.1) go to 30

*           **** check z
            if (fhitb_z(ih).lt.zmin.or.fhitb_z(ih).gt.zmax) go to 40

*           **** check cmp
            cmp = igede_cmp(ihitb_gede(ih))
**            if (cmp.lt.1.or.cmp.gt.3) go to 40
            if (cmp.lt.1.or.cmp.gt.4) go to 40
            if (cmp.eq.1.and.fhitb_z(ih).gt.200.) go to 40


            if (nzf_s.ge.nzfmx_s) then
               write(c,'((a),i8)') 
     +              ' () tfmt2seg: overflowing nzf_s=', nzf_s
               call mewarn(c)
               go to 30
            endif
            nzf_s = nzf_s + 1
            z_s(nzf_s) = fhitb_z(ih)
            hit_s(nzf_s) = ih
            mimp_s(nzf_s) = mimp
            if (mimp.le.0.or.mimp.gt.iq(jmimp+2)) then
               write(c,'((a),6(i8,(a),i8))') 
     +              ' () tfmt2seg: invalid mimp=',mimp,
     +              ' imc=',imc,' iso=',iso,' hit=',ih,
     +              ' imimmh(imc)=',imimmh(imc),
     +              ' nmimmh(imc)=',nmimmh(imc)
               call mewarn(c)
            endif
 40      continue
 30   continue

      if (nzf_s.lt.hit_min) then
         go to 999
      endif

      if (nForward.gt.0.and.nBackward.gt.0) then
         write(c,'((a),i8)') 'tfmt2seg: curl detected for imc=',imc
         call mewarn(c)
         go to 999
      endif

*     **** sort again (to be sure)
      call sortzv(z_s,ind_s,nzf_s,+1,1,0)

*     **** print
#ifdef FULL
      if (diag_level.ge.5) then
         write(*,'(2x,4a6,a8  ,2a9  ,2a8  ,''('',a8  ,'')'',a8  )')
     +     'ihit', 'isl', 'mimp', ' imc ',' z ', ' x0 ', ' y0 ', ' mx ',
     +        ' my ', ' myp ', ' p '

         do ii=1,nzf_s
            ih = hit_s(ind_s(ii))
            mimp = mimp_s(ind_s(ii))
            write(*,'(2x,4i6,f8.2,2f9.3,2f8.5,''('',f8.5,'')'',f8.3)')
     +           ih, ogede_gesl(ihitb_gede(ih)), mimp, omimp_mtra(mimp),
     +           fmimp_z(mimp), fmimp_x(mimp), fmimp_y(mimp),
     +           fmimp_tx(mimp), fmimp_ty(mimp),
     +           fmimp_ty(mimp)/sqrt(1.+fmimp_tx(mimp)**2),
     +           fmimp_p(mimp)
         enddo
      endif
#endif

*     **** mimp statistics
      mimp1 = mimp_s(ind_s(1))
      mimp2 = mimp_s(ind_s(nzf_s))

#ifdef FULL
      if (mimp1.le.0.or.mimp1.gt.iq(jmimp+2)) then
         print *,' () tfmt2seg: invalid mimp1=',mimp1,
     +        ' ind_s(1)=',ind_s(1),
     +        ' nzf_s=',nzf_s,
     +        ' imimmh(imc)=',imimmh(imc),
     +        ' nmimmh(imc)=',nmimmh(imc)
      endif
      if (mimp2.le.0.or.mimp2.gt.iq(jmimp+2)) then
         print *,' () tfmt2seg: invalid mimp2=',mimp2,
     +        ' ind_s(nzf_s)=',ind_s(nzf_s),
     +        ' nzf_s=',nzf_s,
     +        ' imimmh(imc)=',imimmh(imc),
     +        ' nmimmh(imc)=',nmimmh(imc)
      endif
#endif

*     **** store into RSEG table
      call mmbook(jrseg,1)      ! increase row counter
      iseg = nrseg()

*     **** take impact parameter of lowest-z point
      izf = ind_s(1)

      q(qrseg_zf_l()) = fhitb_z(hit_s(ind_s(1)))
      q(qrseg_ze_l()) = fhitb_z(hit_s(ind_s(nzf_s)))
      
*     **** fill the track parameters
      q(qrseg_xf(iseg))  = fmimp_x(mimp1)
      q(qrseg_yf(iseg))  = fmimp_y(mimp1)
      q(qrseg_txf(iseg)) = fmimp_tx(mimp1)
      q(qrseg_tyf(iseg)) = fmimp_ty(mimp1)
      q(qrseg_pf(iseg))  = tfmitoqp(mimp1)

*      **** covariance of track parameters
      do i=1,5
         do j=i,5               ! packing
            i_pack = matrix_pack(i,j)
            q(qrseg_cvf(i_pack,iseg)) = 0. ! nothing known about cov
         enddo
      enddo

*     **** fill track parameters for last point
      q(qrseg_xe(iseg))  = fmimp_x(mimp2)
      q(qrseg_ye(iseg))  = fmimp_y(mimp2)
      q(qrseg_txe(iseg)) = fmimp_tx(mimp2)
      q(qrseg_tye(iseg)) = fmimp_ty(mimp2)
      q(qrseg_pe(iseg))  = tfmitoqp(mimp2)

*     **** covariance of track parameters
      do i=1,5
         do j=i,5               ! packing
            i_pack = matrix_pack(i,j)
            q(qrseg_cve(i_pack,iseg)) = 0. ! nothing known about cov
         enddo
      enddo

*     **** set segment flag
      iq(iqrseg_fit_l()) = 90   ! let 90 stand for unfitted track
      q (qrseg_chi_l()) = -1.

      iq(iqrseg_cmp_l()) = 0

      do 20 ii=1,nzf_s          ! decreasing z order
         izf = ind_s(ii)
         ih = hit_s(izf)
         if (ih.le.0.or.ih.gt.nhitb()) then
            print *,' () tfstseg: izf, nzf_s, ii, ih, iq(jhitb+2)=',
     +           izf, nzf_s, ii, ih, iq(jhitb+2)
            go to 20
         endif
*        **** allocate row for new hit in RHIT table
         call mmbook(jrhit,1)
         rh = nrhit()

         q(qrhit_x_l())  = fmimp_x(mimp_s(izf))
         q(qrhit_y_l())  = fmimp_y(mimp_s(izf))
         q(qrhit_tx_l()) = fmimp_tx(mimp_s(izf))
         q(qrhit_ty_l()) = fmimp_ty(mimp_s(izf))
         q(qrhit_p_l())  = tfmitoqp(mimp_s(izf))

         q(qrhit_q_l()) = fhitb_q(ih)

**       iq(iqrhit_rtra_l()) = it  ! track
         call mmbrel(jhirh,ih,rh) ! relation hitb <-> rhit

         call mmbrel(jrsrh,iseg,rh) ! relation rseg <-> rhit

*        **** set segment assignment
         if (ih.gt.0.and.ih.le.nhitb()) then
            call mmbrel(jrshi,iseg,ih)
         else
            print *,' () tfstseg: invalid ih=', ih
         endif

 20   continue   ! hitf loop

*     **** set relation rtra <-> mtra
**      call mmbrel(jrsmt,iseg,imc)  ! relation swap
      call mmbrel(jmtrs,imc,iseg)

      tfmt2seg = iseg

 50   if (diag_level.ge.1) then
         print *,' **************************************************'
         print *,' *    track prepared, current number=',iseg
         print *,' *                  number of hits=',nzf_s
         print *,' **************************************************'
      endif

 999  end

