*-- Author :    Rainer Mankel   15/01/95

      subroutine tfswim(it,rze,rxe,rce,istat)

*     **** takes the first-point track parameters of track it in
*     **** the track bank & performs transport to z=rze

      implicit none

      integer it  ! track number in track table
      real   rze  ! z value to which track parameters should be swum
      real rxe(5) ! track parameters on output
      real rce(5,5) ! cov of ||
      integer istat ! zero if transport was successful

#include "pris/hgmagnhelp.fi"
#include "arte/Arte.fi"
#include "auto/RTRA.fi"

      integer ifstart, ifend, ii, jj, kk, ll, mm,ndimmag, npoints
      parameter (ndimmag=5)
      double precision dz, za, ze, rrstart, xa(ndimmag), xe(ndimmag),
     +       ca(ndimmag,ndimmag), ce(ndimmag,ndimmag)
      real entry_pt(3), exit_pt(3)
      double precision entry_z, exit_z, sum

      double precision f (ndimmag,ndimmag), f1(ndimmag,ndimmag),
     +       f2(ndimmag,ndimmag)
      double precision x_border(ndimmag)

#include "pris/zmpack.fi"

#include "arte/Arte.ff"
#include "auto/RTRA.ff"

*     **** local statement functions
      entry_z() = dble( entry_pt(3) )
      exit_z()  = dble( exit_pt (3) )

      istat = 0

*     **** start vector
      za = frtra_zf(it)
      ze = rze

      xa(1) = frtra_xf(it)
      xa(2) = frtra_yf(it)
      xa(3) = frtra_txf(it)
      xa(4) = frtra_tyf(it)
      xa(5) = frtra_pf(it)

*     **** unpack covariance of track parameters
      do ii=1,5
       do jj=1,5
        ca(ii,jj) = frtra_cvf(matrix_pack(ii,jj),it)
       enddo
      enddo

*     **** step 1: determine case flags
*            ifstart=1 : starting point outside magnet
*                    2 :    ||          inside    ||
*            ifend  =1 : exit     point outside   ||
*                    2 :    ||          inside    ||

      dz = ze - za

      if (za.gt.(zposmg+rmag).and.ze.gt.(zposmg+rmag)) then
       ifstart = 1
       ifend   = 1
      else
*      **** check more precisely if starting point is inside magnet
       rrstart = (za - zposmg)**2 + xa(1)**2
*      **** start flag
       if (rrstart.gt.dble(rmag**2)) then
        ifstart = 1  ! starting point outside magnet
       else
        ifstart = 2  ! starting point inside  magnet
       endif
*      **** search intersection
       call tfmgla(ifstart,xa,za,npoints,entry_pt,exit_pt)
*      **** check endpoint
       if (npoints.gt.0) then
        if (ifstart.eq.1) then
         if (ze.gt.entry_z().or.za.lt.entry_z()) then
          ifend= 1   ! end point outside magnet
         else
          ifend = 2  ! end point inside  magnet
         endif
        else
         if (ze.lt.exit_z()) then
          ifend= 1   ! end point outside magnet
         else
          ifend = 2  ! end point inside  magnet
         endif
        endif
       else
*       **** no further intersection with magnet path
*       **** assume track stays in medium
        ifend = ifstart
       endif
      endif

*     **** convert Q/p to kappa if needed
      if (ifstart.ne.1.or.ifend.ne.1) call tfp2qpt(xa,ca,xa,ca)

*     **** step 2: transport vector & fill derivative matrix f
      if (ifstart.eq.1.and.ifend.eq.1) then
*      **** start/end outside magnet
*      **** ignore possibility of WHOLE MAGNET in between
       xe(1) = xa(1) + dz * xa(3)
       xe(2) = xa(2) + dz * xa(4)
       xe(3) = xa(3)
       xe(4) = xa(4)
       xe(5) = xa(5)

*      **** f matrix
       do ii=1,5
        do jj=1,5
         if (ii.eq.jj) then
          f(ii,jj) = 1.
         elseif ((ii.eq.1.and.jj.eq.3).or.(ii.eq.2.and.jj.eq.4)) then
          f(ii,jj) = dz
         else
          f(ii,jj) = 0.
         endif
        enddo
       enddo
      elseif (ifstart.eq.2.and.ifend.eq.2) then
*      **** track fully contained in magnet
       call tfthel(ndimmag,xa(1),dz,xe(1),f,istat)
       if (istat.lt.0) then
        print *,' tfswim: transport impossible (hel) '
        go to 999
       endif
      else
*      **** split transport into inside & outside pieces
       if (ifstart.eq.1.and.ifend.eq.2) then
*       **** from outside to inside
        call tftlin(ndimmag,xa(1),entry_z()-za,x_border,f1)
        call tfthel(ndimmag,x_border,ze-entry_z(),xe(1),f2,istat)
        if (istat.lt.0) then
         print *,' tfswim: transport impossible (lin/hel) '
         go to 999
        endif

       elseif (ifstart.eq.2.and.ifend.eq.1) then
*       **** from inside to outside
        call tfthel(ndimmag,xa(1),exit_z()-za,x_border,f1,istat)
        if (istat.lt.0) then
         print *,' tfswim: transport impossible (hel/lin) '
         go to 999
        endif

        call tftlin(ndimmag,x_border,ze-exit_z(),xe(1),f2)
       endif

*      **** combine derivative matrices
       do ii=1,ndimmag
        do jj=1,ndimmag
         sum = 0.
         do kk=1,ndimmag
          sum = sum + f2(ii,kk) * f1(kk,jj)
         enddo
         f(ii,jj) = sum
        enddo
       enddo
      endif

*     **** step 3: propagate cov matrix
      do ii=1,5
       do jj=1,5
        sum = 0.
        do ll=1,5
         do mm=1,5
          if (ll.eq.mm.and.(ll.eq.3.or.ll.eq.4)) then
           sum = sum + f(ii,ll) * ca(ll,mm) * f(jj,mm)
          else
           sum = sum + f(ii,ll) * ca(ll,mm) * f(jj,mm)
          endif
         enddo
        enddo
        ce(ii,jj) = sum
       enddo
      enddo

*     **** convert back kappa to Q/p if needed
      if (ifstart.ne.1.or.ifend.ne.1) call tfqpt2p(xe,ce,xe,ce)

*     **** convert to single precision
      do ii=1,ndimmag
       rxe(ii) = sngl(xe(ii))
       do jj=1,ndimmag
        rce(ii,jj) = sngl( ce(ii,jj) )
       enddo
      enddo

 999  end



