//--*-C++-*--
#ifndef RCXATTRIBUTE_H
#define RCXATTRIBUTE_H

// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) decay length added (min_dz_from_wire)
// 06 Aug 2000 (DD) selection flag for J/Psi electrons (edora),
//                  based on D.Samtleben/T.Oest selection:
//                  brems || otr || rich
//
// *****************************************************************************

#include <stdio.h>
#include "arte/arte.hh"
#include "reco/rcXmessage.h"

class attribute {
 public:
  attribute();  
  ~attribute() {;}

  /// check attributes, return >0 if attribute not passed
  int   checkAttributes(ArtePointer<RTRA> t);

  /// Momentum cut 
  float minPt,minP,maxP;

  /// Track quality
  float maxChi;

  /// VDS quality
  int   minNhits, minPhits, minHSI; 
  
  /// ITR/OTR quality
  int   minMClayers, minPClayers, minTClayers, minHOU, minHIN, minHIO;

  /// RICH particle id
  float  maxElectronR2P2, maxMuonR2P2, maxPionR2P2, maxKaonR2P2, maxProtonR2P2;
  float  minLRE,          minLRMU,     minLRPI,     minLRK,      minLRP;
  /// RICH ring quality
  float  minRichRadius, maxRichRadius, minRichSN, minHRI;
  /// Exclude RICH bands
  float  maxLRE, maxLRMU, maxLRPI, maxLRK, maxLRP;

  /// ECAL quality
  float maxEcalEnergy, minEcalWid, minEoP, maxEoP;
  int   minHCA;
  /// ECAL particle id
  float minLEE, minLEMI, minLEH, maxLEE, maxLEMI, maxLEH;

  /// MUON quality
  int minHMU;

  /// Combinations
  int minHCAMU; // hca+hmu
  int minHRIOU; // hri+hin
  int minHRIIN; // hri+hou

  /// MC information
  int lund, mother;
  ArtePointer<TOPP> topp;
   
  /// name
  char name[20];
  inline void setName(char *n) { if (sizeof(n)>19) return; sprintf(name,"%s",n); }

  /// fit type
  int fit;

  /// mass range for reconstructed decays, mass for nondecaying tracks
  float minMass, maxMass, mass;

  /// Absolute vertex position
  float minVertexZ, maxVertexZ;

  /// Minimum downstream decay length
  float min_dz_from_wire;

  /// bremsstrahlung
  int recoverBremsstrahlung;

  /// Use vds segment only once in track list for this particle
  int useVdsOnce;

  /// Cloning of the same particle type, default clone=1, for 2 pi+ clone=2, ...
  int clone;

  /// Maximum number of track candidates
  int maxCandidate;

  /// Special algorithm to select electrons
  /// by T.Oest/D.Samtleben: (brems || otr || rich)
  int eDora;

  /// Special algorithm to select muons
  /// by K.Reeves
  int muKen;

};

#endif
