//--*-C++-*--
#ifndef RCXVERTEX_H
#define RCXVERTEX_H

// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) search for primary vertex and decay wire
//
// *****************************************************************************

#include "arte/arte.hh"
#include "CLHEP/Matrix/Vector.h"
#include "vector.h"

#include "reco/rcXdecay.h"


class vertexmass {
 private:

  struct TRACK {
    ArtePointer<RTRA> rtra;
    float             x[3];
    float             n[3];
    ArtePointer<TOPP> topp;
  };
  vector<TRACK>*  track_list;

  /// description of vertexing+mother
  vertexAttrib      vattrib;
  float             Mass, Merr;
  /// Decay RTRA
  ArtePointer<RTRA> decay_rtra;
  /// Decay MTRA
  ArtePointer<MTRA> decay_mtra;
  /// Decay MVER
  ArtePointer<MVER> decay_ever;
  float             chi2;
  float             a[3][3], v[3], Cov[15];
  double            par[5];


  /// chi^2 method (P.Schlein)
  void vtchi();
  /// Kalman filtering method (T.Lohse)
  void vtfit();
  /// Mass constraint fit (T.Lohse)
  void vtmass();

  /// find primary vertex for given decay parameters
  ArteTable<RVER>::iterator findRVER(float *fpar);
  /// find primary vertex (wire) and check decay length
  int checkPrimaryVertex(float *fpar);
  /// find target using current v[3] and track_list
  vector<ArtePointer<GTAR> >::iterator findGTAR(float *par);
  /// list of used wires
  vector<ArtePointer<GTAR> > *wire_list;
  /// wire from which decay originates
  ArtePointer<GTAR> decay_wire;
  /// find wires in use (close to beam)
  void findUsedWires();
  /// primary vertex for this decay (RC)
  ArtePointer<RVER> v0_rver;
  /// primary vertex for this decay (MC)
  ArtePointer<MVER> v0_mver;
  int    findPrimaryVertex();

 public:
  /// constructor
  vertexmass();

  /// adds a track to the list
  void addVertexTrack(ArtePointer<RTRA> rtra, ArtePointer<TOPP> topp=0);

  /// get a track for the list
  inline ArtePointer<RTRA> getVertexTrack(int i) { return (*track_list)[i].rtra; }

  /// removes last track from list
  void removeLastVertexTrack();

  /// removes last track from list
  inline void removeAllVertexTracks() { if (track_list->size()) track_list->clear(); }

  /// get number of tracks in bank
  inline int  getNVertexTrack() { return track_list->size(); }

  /// make vertex fit
  int  fitVertex();

  /// print tracks included in fit
  void printVertex();

  /// get chi^2
  inline float &vertexChi2() { return chi2; }

  /// vertex attributes
  inline void  setVertex(vertexAttrib &va)          { vattrib=va;     }
  inline const vertexAttrib &getVertAttrib()  const { return vattrib; }

  /// get vertex coordinates
  inline float &vertexX(int i) { return v[i]; }

  /// get covariance matrix
  inline float &vertexCov(int i) { return Cov[i]; } 

  /// find closest wire that reconstructed vertex points to 
  /// and make dz cut
  int  checkWire(float *fpar);

  /// mother
  inline float             &massX() { return Mass; }
  inline float             &merrX() { return Merr; }
  inline ArtePointer<RTRA> &rtraX() { return decay_rtra; }
  inline ArtePointer<MTRA> &mtraX() { return decay_mtra; }
  inline ArtePointer<MVER> &mverX() { return decay_ever; }

  /// Booking
  void bookRVER();

  /// Get vertex, primary vertex and mtra
  void   findVertexMC();
  inline ArtePointer<RVER> &rverV0() { return v0_rver; }
  inline ArtePointer<MVER> &mverV0() { return v0_mver; }

  /// Wire(s)
  ArtePointer<GTAR> getDecayWire() { return decay_wire; }

};


#endif
