
// *****************************************************************************
//
// source:    rcSelectVZERO2.C
//
// type:      source code
//
// created:   19 Jul 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcSelectVZERO2
//
// called by: rcClassVZERO2
//
// purpose:   select events with V0 candidate (Denis)
//
// hints:     rc = 0x00: nothing
//                 0x01: Ks
//                 0x02: K892
//                 0x04: Phi
//                 0x08: D0
//
// changes:
// 19 Jul 2000 (AG) creation
// 21 Jul 2000 (AG) included code from Ignacio Arinyo (K0sSelect.C)
// 25 Jul 2000 (AG) corrected bug
// 26 Jul 2000 (AG) included Denis' selection
// 27 Jul 2000 (AG) added D0()
// 05 Aug 2000 (AG) removed printout
// 06 Aug 2000 (AG) added selectB();
// 07 Aug 2000 (AG) selectB() moved to rcSelectDVTX.C
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ARTE include
#include "arte/arte.hh"
#include "farm/farm.hh"

// externals
extern int FADEB;

// prototypes
int copyRTRItoRTRA( int algorithm );

int selectKs();
int selectK892();
int selectPhi();
int selectD0();

//
// function code
//
int
rcSelectVZERO2()
{
  ARTE_mark( "rcSelectVZERO2 0", 0 );
  
  //
  // Denis ********************************************************************
  //
  copyRTRItoRTRA(0);
  int ks   = selectKs();
  int k892 = selectK892();
  int phi  = selectPhi();
  int d0   = selectD0();

  // print
  if( FADEB ){
    cout << " SELECT: VZERO2 " 
         << "ks="   << ks   << " "
         << "k892=" << k892 << " "
         << "phi="  << phi  << " "
         << "d0="   << d0   << " "
         << endl;
   }
  
  //
  // final return code
  //
  int rc = 0;

  // Denis
  if( ks   ) rc |= 0x01; // Ks
  if( k892 ) rc |= 0x02; // K892
  if( phi  ) rc |= 0x04; // Phi
  if( d0   ) rc |= 0x08; // D0

  // print
  if( FADEB ) cout << " SELECT: VZERO2=0x" << hex << rc << dec << endl; 

  ARTE_mark( "rcSelectVZERO2 1", 0 );
  return rc;
}
