
// *****************************************************************************
//
// source:    Selection of B->J/Psi X, requires reconstructed
//            J/Psi and looks for a charge or Ks that points to
//            the same vertex
//
// type:      source code
//
// created:   04 Aug 2000
//	
// author:    Denis Dujmic
//
// changes:
//  20 Aug 2000 (DD) debugging rhp histograms only in offline mode
//  21 Aug 2000 (DD) bug fix
//
// *****************************************************************************
#include "reco/rcX.h"
#include "reco/rcXkinema.h"
#include "reco/rcXsegment.h"
#include "rhp.h"
#include "dq/DQ.h"

int B__JPsi_X_RC();
int B__JPsi_X_MCeff(vertexAttrib &vattrib, vertexmass &vert, int total);
int maximizeRTRA();




/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////  REAL RECO   /////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////



// 
//  select displaced J/Psi with at least one additional prong
// 
int B__JPsi_X_RC() {

  int verb =0;
  int total=0;

  if (verb) cout<<">>>>>>>>>>>>>>>>>  B select <<<<<<<<<<<<<<<<<<"<<endl;

  static vertexmass   vert;
  static vertexAttrib vattrib;
  static attribute    jpsi;
  static attribute    x, ks;
  static bool         first=true;
  if (first) {
    ///////////////////////////// ALL CUTS HERE ///////////////////////////////
    // -- description of vertex fitting
    vattrib.rlevel           = 1;      // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vattrib.momentum_flag    = 0;      // momentum in fit?
    vattrib.max_vertex_chi2  = 3.0 ;   // upper limit for vertex acceptance 
    vattrib.verb             = 0;      // vertexing printout level
    vattrib.min_dz_from_wire = 0.25;   // minimum decay distance for B (cm)
    vattrib.check_wire       = 1;      // check primary vertex/wire
    vert.setVertex(vattrib);           // set

    // -- description of J/Psi
    jpsi.fit     = 2;                       // short decay
    jpsi.minMass = 2.5;                     // min reco mass
    jpsi.maxMass = 3.5;                     // max reco mass
    
    // -- description of candidates for common vertexing with jpsi
    x.fit        = 0; // charged track
    x.minHSI     = 5; // >>> require only vds <<< !!!

    // -- description of ks
    ks.fit     =    3; // long decay
    ks.minMass = .470; // min reco mass
    ks.maxMass = .530; // max reco mass
    ///////////////////////////// ALL CUTS HERE ///////////////////////////////
    first=false;
  }

  // -- find jpsi's and other candidates
  static vector<ArtePointer<RTRA> > jpsi_list(10);
  static vector<ArtePointer<RTRA> > x_list(100);
  static vector<ArtePointer<RTRA> >::iterator jpsi_it;
  static vector<ArtePointer<RTRA> >::iterator x_it;
  if (jpsi_list.size()) jpsi_list.clear();
  if (x_list.size())    x_list.clear();

  //static ArtePointer<MTRA> mB;
  //for (ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin();
  //irtra!=ArteTable<RTRA>::end(); ++irtra) {
  //if (!jpsi.checkAttributes(*irtra)) {
  //mB=(*(*irtra)->mtra.begin())->btra;
  //}
  //}


  for (ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin();
       irtra!=ArteTable<RTRA>::end(); ++irtra) {
    if (!jpsi.checkAttributes(*irtra)) {
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1000 ),   0., 1.); // jpsi found
      float fpar[]={(*irtra)->xf,(*irtra)->yf, (*irtra)->zf, (*irtra)->txf, (*irtra)->tyf};
      vert.checkWire(fpar);  // find used wire
      if ((*irtra)->zf-vert.getDecayWire()->z>vattrib.min_dz_from_wire)
	rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1000 ),  1., 1.); // jpsi with proper displacement
      float zjp=vert.vertexX(2);
      rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1005 ), zjp, 1.);
      jpsi_list.push_back(*irtra);
    }
    if (!x.checkAttributes(*irtra))  {
      //int lund=abs((*(*irtra)->mtra.begin())->topp->lund);
      //if (((*(*irtra)->mtra.begin())->btra==mB  ||
      //(*(*irtra)->mtra.begin())->btra->btra==mB &&
      //(abs((*(*irtra)->mtra.begin())->btra->topp->lund)==113 ||
      //abs((*(*irtra)->mtra.begin())->btra->topp->lund)==313)
      //)
      //&& (lund==211 || lund==321))
      x_list.push_back(*irtra);
    }
    if (!ks.checkAttributes(*irtra)) x_list.push_back(*irtra);
  }
  if (verb>1) {
    cout<<"Total Jpsi="<<jpsi_list.size();
    for (jpsi_it=jpsi_list.begin(); jpsi_it!=jpsi_list.end(); ++jpsi_it)  
      cout<<" "<<(*jpsi_it)<<" ";
    cout<<endl;
    cout<<"Total tracks="<<x_list.size();
    for (x_it=x_list.begin(); x_it!=x_list.end(); ++x_it)  
      cout<<" "<<(*x_it)<<" ";
    cout<<endl;
  }
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1018 ), float(jpsi_list.size()), 1.); 
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1019 ),    float(x_list.size()), 1.); 

  if (jpsi_list.size() && x_list.size()) {

    // -- search for multiprong vertex
    for (jpsi_it=jpsi_list.begin(); jpsi_it!=jpsi_list.end(); ++jpsi_it) {    

      // -- intialize search
      vert.removeAllVertexTracks();
      ArtePointer<RVER> ever=(*jpsi_it)->ever;

      // -- add J/Psi leptons to vertex track bank
      for (ArteRelationIterator<RREL> irrel=ever->rrel.begin();
	   irrel!=ever->rrel.end(); ++irrel) {
	vert.addVertexTrack((*irrel)->rtra);
      }
      if (vert.getNVertexTrack()!=2) { 
	cout<<"B__JPsi_X_RC: something wrong, jpsi without 2 tracks"<<endl; 
	continue; 
      }
      vert.fitVertex();
      float chi0=vert.vertexChi2();
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1002 ), chi0, 1.);

      // -- add vertex candidates into vertex track bank
      static vector<ArtePointer<RTRA> > prong_list(10);
      if (prong_list.size()) prong_list.clear();
      for (x_it=x_list.begin(); x_it!=x_list.end(); ++x_it) {
	// -- check if candidate uses any segment already used by jpsi leptons
	if (segment::TrackDiff((*x_it),(*jpsi_it))) continue;
	// -- add track and check vertex
	vert.addVertexTrack(*x_it);
	int   vflg=vert.fitVertex();
	float chi2=vert.vertexChi2();
	rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1003 ), chi2, 1.);
	// -- accept if:
	//        - chi2 improves
	//        - chi2 small
	if (chi2<chi0 || chi2<vattrib.max_vertex_chi2) { 
	  rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1004 ), chi2, 1.);
	  prong_list.push_back(*x_it); 
	  chi0=chi2; 
	}
	vert.removeLastVertexTrack();
      }

      // -- book multiprong vertex (RVER) and fill dq histos
      if (!prong_list.size()) continue;
      for (x_it=prong_list.begin(); x_it!=prong_list.end(); ++x_it) 
	vert.addVertexTrack((*x_it));
      vert.fitVertex();
      float zv=vert.vertexX(2);
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1011 ), zv, 1.);
      float dz0=vert.vertexX(2) - vert.getDecayWire()->z;
      if (dz0<vattrib.min_dz_from_wire) continue;
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1000 ),   2., 1.); // event selected
      float nt=float(vert.getNVertexTrack());
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1001 ),   nt, 1.);
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1006 ),   zv, 1.);
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1014 ),  dz0, 1.);
      float wx=vert.getDecayWire()->x;
      float wy=vert.getDecayWire()->y;
      float wz=vert.getDecayWire()->z;
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1015 ),    wz,     1.);
      rhp_hfill(DQ_ID( COMP_RECO,LVL_REC, 1016 ),  wx, wy, 1.);
      vert.bookRVER();
      total++;
      if (verb) {
	ArtePointer<RVER> Rver(ArteTable<RVER>::size());
	cout<<"found vertex="<<Rver<<": "<<Rver->xv<<","<<Rver->yv<<","<<Rver->zv<<endl;
	cout<<"RC: prongs found="<<Rver->nt<<endl;
      }
    }

  }
  
  // -- evaluate reco efficiency (MC only)
  if (ArteTable<MTRA>::size()) B__JPsi_X_MCeff(vattrib, vert, total);

  if (verb) cout<<">>>>>>>>>>>>>>>>>  B select <<<<<<<<<<<<<<<<<<"<<endl;

  return total;
}




//
//  MC: Monitor reco efficiency for  J/Psi + prongs 
//
int B__JPsi_X_MCeff(vertexAttrib &vattrib, vertexmass &vert, int total) {
  if (!ArteTable<MTRA>::size()) return 1;
  int verb=0;

#ifdef USE_ROOT_DD
  static bool first=true;
  if (first) {
    first=false;
    rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1007 ),"B selection MC: Event efficiency",  6,  -.5,  5.5 ); 
    rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1008 ),"B selection MC: B prongs in vds", 10,  -.5,  9.5 ); 
    rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1009 ),"B selection MC: prongs/vertex", 10,  -.5,  9.5 ); 
    rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1010 ),"B selection MC: B prongs in sel. events", 10,  -.5,  9.5 );
    rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1012 ),"B selection MC: B vertex resoltion", 50,  -1,  1 ); 
    rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1013 ),"B selection MC: Decay length from MC v0", 50,  -1,  5 ); 
    rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1017 ),"B selection MC: Decay length (MC truth)", 50,0,5); 
    rhp_hbook1( DQ_ID( COMP_RECO,LVL_REC, 1020 ),"B selection MC: MC v0 - RC v0 (MC)", 50,  -5,  5 ); 
  }
#endif

  // -- check if J/Psi from B+, B0 exists
  ArteTable<MTRA>::iterator imtra=ArteTable<MTRA>::begin();
  ArteTable<MTRA>::iterator Bmtra=ArteTable<MTRA>::end();
  for (; imtra!=ArteTable<MTRA>::end(); ++imtra) {
    if ((*imtra)->topp->lund==443 &&
	(abs((*imtra)->btra->topp->lund)==521 || 
	 abs((*imtra)->btra->topp->lund)==511)) { 
      Bmtra=(*imtra)->btra; break; 
    }
    if ((*imtra)->topp->lund==443 &&
	(abs((*imtra)->btra->btra->topp->lund)==521 ||
	 abs((*imtra)->btra->btra->topp->lund)==511)) {
      Bmtra=(*imtra)->btra->btra; break;
    }
  }
  if (imtra==ArteTable<MTRA>::end()) return 0;
  ArteTable<MTRA>::iterator JPmtra=imtra;

  // -- check J/Psi decays to leptons
  int vdsflg[]={0,0}, i=0;
  for (ArteRelationIterator<MTRA> jmtra=(*JPmtra)->etra.begin();
       jmtra!=(*JPmtra)->etra.end(); ++jmtra) {
    int lund=abs((*jmtra)->topp->lund);
    if (lund!=11 && lund!=13) return 2;
    if (i==2)                 return 2;
    for (ArteRelationIterator<RTRA> irtra=(*jmtra)->rtra.begin();
	 irtra!=(*jmtra)->rtra.end(); ++irtra) {
      if ((*irtra)->hsi>0) vdsflg[i++]=1;
    }
  }
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1007 ), 0., 1.);

  // -- check if both leptons have reconstructed vds
  if (!vdsflg[0] || !vdsflg[1]) return 3;
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1007 ), 1., 1.);

  // -- check if booked in rtra (vertexing)
  if (JPmtra->rtra.begin()==JPmtra->rtra.end()) return 4;
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1007 ), 2., 1.); 

  // -- check if at least one additional prong with vds points to J/Psi vertex
  ArteRelationIterator<MTRA> jmtra=Bmtra->etra.begin();
  static vector<ArtePointer<MTRA> > prong_list(10); 
  if (prong_list.size()) prong_list.clear();
  for (; jmtra!=Bmtra->etra.end(); ++jmtra) {
    // -- reject jpsi
    if ((*jmtra)->topp->lund==443 || (*jmtra)->topp->lund==20443) continue;
    // -- check for nondecaying daughters
    int lund=abs((*jmtra)->topp->lund);
    if (lund==11 || lund==13 || lund==211 || lund==321 || lund==2212) {
      ArteRelationIterator<RTRA> irtra=(*jmtra)->rtra.begin();
      for (; irtra!=(*jmtra)->rtra.end(); ++irtra) if ((*irtra)->hsi>0) { prong_list.push_back(*jmtra); break; }
    }
    // -- check for Ks
    else if (lund==310) {
      for (ArteRelationIterator<MTRA> ipi=(*jmtra)->etra.begin();
	   ipi!=(*jmtra)->etra.end(); ++ipi) {
	if (abs((*ipi)->topp->lund)==211) 
	  for (ArteRelationIterator<RTRA> irtra=(*ipi)->rtra.begin(); 
	       irtra!=(*ipi)->rtra.end(); ++irtra) 
	    if ((*irtra)->hsi>0) {  prong_list.push_back(*ipi); break; };
      }
    }
    // -- check for resonance
    else if ((*jmtra)->flag==2) {
      ArteRelationIterator<MTRA> kmtra=(*jmtra)->etra.begin();
      for (; kmtra!=(*jmtra)->etra.end(); ++kmtra) {
	ArteRelationIterator<RTRA> irtra=(*kmtra)->rtra.begin();
	for (; irtra!=(*kmtra)->rtra.end(); ++irtra) if ((*irtra)->hsi>0) {  prong_list.push_back(*kmtra); break; }
      }
    }
  }
  int nprong=prong_list.size()+2;
  if (nprong==2) return 5;
  if (verb) cout<<"MC: prongs found = "<<prong_list.size()<<endl;
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1007 ),            3., 1.);  

  // -- check if real decay length (dz) passes cut
  float dz0_mc =(*JPmtra)->bver->z - (*Bmtra)->bver->z;
  if (dz0_mc>vattrib.min_dz_from_wire) {
    rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1007 ), 4., 1.);
    rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1008 ), float(nprong), 1.); 
  }


  // -- check if event selected
  if (!total) return 7;
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1007 ), 5., 1.); 
  int sprong=vert.getNVertexTrack();
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1009 ), float(sprong), 1.); 
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1010 ), float(nprong), 1.); 

  // -- plot vertex: z_RC - z_MC (vertex resolution)
  float dz= vert.vertexX(2) - (*(*Bmtra)->ever.begin())->z;
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1012 ), dz, 1.); 

  // -- plot vertex: z_RC - v0_MC
  float dz0_rc= vert.vertexX(2)   - (*Bmtra)->bver->z;
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1013 ), dz0_rc, 1.); 
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1017 ), dz0_mc, 1.);
  float dz0_mcrc =  (*Bmtra)->bver->z - vert.getDecayWire()->z;
  rhp_hf1( DQ_ID( COMP_RECO,LVL_REC, 1020 ), dz0_mcrc, 1.);

  return 0;
}




// 
//  returns number of vertices with a kaon
// 
int B__K(ArteTable<RVER>::iterator iv1, ArteTable<RVER>::iterator iv2) {
  int ret=0;
  ////////// cut //////////
  float minLRK=1e-4;
  ////////// cut //////////
  for (ArteTable<RVER>::iterator irver=iv1; irver!=iv2; ++irver) {
    int nk=0;
    for (ArteRelationIterator<RREL> irrel=(*irver)->rrel.begin();
	 irrel!=(*irver)->rrel.end(); ++irrel) {
      if ((*irrel)->rtra->lrk>minLRK) nk++;
      if ((*irrel)->rtra->fit%100==3) nk++;
    }
    rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1021 ),float(nk), 1.);
    if (nk) {
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1000 ), 3., 1.); // kaon found
      ret++;
    }
  }
  return ret;
}


//
// returns 1 if good vertex, 2 if kaon in vertex, 0 otherwise
//
int
selectB()
{
  ARTE_mark( "rcXB selection 0", 0 );
  int ret=0;
  maximizeRTRA(); // use higer likelihood

  ArteTable<RVER>::iterator iv1=ArteTable<RVER>::end();
  if (B__JPsi_X_RC())       ret=1;
  ArteTable<RVER>::iterator iv2=ArteTable<RVER>::end();
  if (ret && B__K(iv1,iv2)) ret=2; 
  //else                      ret=0;

  ARTE_mark( "rcXB selection 1", 0 );
  //cout<<"selectB ret="<<ret<<endl;
  return ret;
}


//
//
//
void 
stopB() 
{
  ;
}
