
// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 04 Aug 2000 (DD) code polishing
// 06 Aug 2000 (DD) quality histograms
//
// *****************************************************************************

#include "reco/rcX.h"
#include "reco/rcXkinema.h"
#include "rhp.h"
#include "dq/DQ.h"


rcX *rcK892_MC1=0;
rcX *rcK892_MC2=0;
rcX *rcK892_RC1=0;
rcX *rcK892_RC2=0;

int K892__Piplus_Kminus_MC();
int K892__Piminus_Kplus_MC();
int K892__Piplus_Kminus_RC();
int K892__Piminus_Kplus_RC();
int maximizeRTRA();

//
// Main K* reco
//
int 
K892()
{
  if (ArteTable<MTRA>::size()) {
    K892__Piplus_Kminus_MC();
    K892__Piminus_Kplus_MC();
  }
  else {
    K892__Piplus_Kminus_RC();
    K892__Piminus_Kplus_RC();
  }
  return 0;
}

//
//
//
int
K892__Piplus_Kminus_MC()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  K892_MC1 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcK892_MC1) {
    cout<<"initializing K892"<<endl;
    rcK892_MC1=new rcX("K892_1", "K892 -> Pi+K-", verb);
    decay k892;
    k892.fit           = 2;       // = short decay
    k892.book_tables   = 1;       // 1/0
    k892.rlevel        = 2;       // 0=full, 1=semi, 2=mtra
    k892.verb          = verb;    // printout level
    rcK892_MC1->setMother(k892);  // set
    vertexAttrib vk892;
    vk892.rlevel           = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vk892.topp             = kinema::findTopp(313); // pointer to mother
    vk892.momentum_flag    = 1;     // momentum in fit?
    vk892.max_vertex_chi2  = 10.0;  // limit for vertex acceptance 
    vk892.verb             = verb;  // printout level
    vk892.min_mass         = 0.5;   // minimum mass
    vk892.max_mass         = 1.5;   // maximum mass
    rcK892_MC1->setVertex(vk892);   // set

    attribute pi1;
    pi1.lund=211;
    pi1.topp=kinema::findTopp(pi1.lund);
    pi1.mother=313;
    pi1.mass=pi1.topp->mass;
    pi1.minP=0;
    pi1.maxP=200;
    rcK892_MC1->addDaughter(pi1);

    attribute k1;
    k1.lund=-321;
    k1.topp=kinema::findTopp(k1.lund);
    k1.mother=313;
    k1.mass=k1.topp->mass;
    k1.minP=-200;
    k1.maxP=0;
    rcK892_MC1->addDaughter(k1);  
  }
  rcK892_MC1->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  K892_MC1 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}

//
//
//
int
K892__Piminus_Kplus_MC()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  K892_MC2 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcK892_MC2) {
    cout<<"initializing K892"<<endl;
    rcK892_MC2=new rcX("K892_2", "K892 -> Pi-K+", verb);
    decay k892;
    k892.fit           = 2;      // = short decay
    k892.book_tables   = 1;      // 1/0
    k892.rlevel        = 2;      // 0=full, 1=semi, 2=mtra
    k892.verb          = verb;   // printout level
    rcK892_MC2->setMother(k892); // set
    vertexAttrib vk892;
    vk892.rlevel           = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vk892.topp             = kinema::findTopp(313); // pointer to mother
    vk892.momentum_flag    = 1;     // momentum in fit?
    vk892.max_vertex_chi2  = 10.0;  // limit for vertex acceptance 
    vk892.verb             = verb;  // printout level
    vk892.min_mass         = 0.5;   // minimum mass
    vk892.max_mass         = 1.5;   // maximum mass
    rcK892_MC2->setVertex(vk892);   // set

    attribute pi1;
    pi1.lund=-211;
    pi1.topp=kinema::findTopp(pi1.lund);
    pi1.mother=313;
    pi1.mass=pi1.topp->mass;
    pi1.minP=-200;
    pi1.maxP=0;
    rcK892_MC2->addDaughter(pi1);

    attribute k1;
    k1.lund=321;
    k1.topp=kinema::findTopp(k1.lund);
    k1.mother=313;
    k1.mass=k1.topp->mass;
    k1.minP=0;
    k1.maxP=200;
    rcK892_MC2->addDaughter(k1);  
  }
  rcK892_MC2->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  K892_MC2 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}

/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////  REAL RECO   /////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////

int
K892__Piplus_Kminus_RC()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  K892_RC1 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcK892_RC1) {
    cout<<"initializing K892"<<endl;
    rcK892_RC1=new rcX("rec_K892a", "K892 -> Pi+K-", verb);
    decay k892;
    k892.rlevel        = 0;         // 0=full, 1=semi, 2=mtra
    k892.fit           = 102;       // = short decay
    k892.book_tables   = 1;         // 1/0
    k892.root_output   = 1;         // make root file or not
    k892.verb          = verb;      // printout level
    rcK892_RC1->setMother(k892);    // set
    vertexAttrib vk892;
    vk892.rlevel           = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vk892.topp             = kinema::findTopp(313); // pointer to mother
    vk892.max_vertex_chi2  = 10.0;  // limit for vertex acceptance 
    vk892.verb             = verb;  // printout level
    vk892.min_mass         = 0.5;   // minimum mass
    vk892.max_mass         = 1.5;   // maximum mass
    rcK892_RC1->setVertex(vk892);   // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(211);
    pi1.mass         = pi1.topp->mass;
    pi1.minP         = 0;
    pi1.maxP         = 200;
    pi1.minPt        = 0.15;
    pi1.minLRPI      = .001;
    pi1.maxCandidate = 10;
    pi1.minHRI       = 1;
    pi1.minHSI       = 6;
    pi1.minHIO       = 18;
    pi1.useVdsOnce   = 1;
    rcK892_RC1->addDaughter(pi1);

    attribute k1;
    k1.topp         = kinema::findTopp(-321);
    k1.mass         = k1.topp->mass;
    k1.minP         = -200;
    k1.maxP         = 0;
    k1.minPt        = 0.3;
    k1.minLRK       = .001;
    k1.maxCandidate = 10;
    k1.minHRI       = 1;
    k1.minHIO       = 18;
    k1.minHSI       = 6;
    k1.useVdsOnce   = 1;
    rcK892_RC1->addDaughter(k1);

    if (verb) rcK892_RC1->printDaughters();
  }
  rcK892_RC1->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  K892_RC1 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}

//
//
//
int
K892__Piminus_Kplus_RC()
{
  int verb=0;
  
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  K892_RC2 <<<<<<<<<<<<<<<<<<<"<<endl;
  if (!rcK892_RC2) {
    cout<<"initializing K892"<<endl;
    rcK892_RC2=new rcX("rec_K892b", "K892 -> Pi-K", verb);
    decay k892;
    k892.rlevel        = 0;         // 0=full, 1=semi, 2=mtra
    k892.fit           = 102;       // = short decay
    k892.book_tables   = 1;         // 1/0
    k892.root_output   = 1;         // make root file or not
    k892.verb          = verb;      // printout level
    rcK892_RC2->setMother(k892);    // set
    vertexAttrib vk892;
    vk892.rlevel           = 2;     // 0=no, 1=chi2, 2=kalman, 3=kalman+mass
    vk892.topp             = kinema::findTopp(313); // pointer to mother
    vk892.max_vertex_chi2  = 10.0;  // limit for vertex acceptance 
    vk892.verb             = verb;  // printout level
    vk892.min_mass         = 0.5;   // minimum mass
    vk892.max_mass         = 1.5;   // maximum mass
    rcK892_RC2->setVertex(vk892);   // set

    attribute pi1;
    pi1.topp         = kinema::findTopp(-211);
    pi1.mass         = .139567;
    pi1.minP         = -200;
    pi1.maxP         = 0;
    pi1.minPt        = 0.15;
    pi1.minLRPI      = .001;
    pi1.maxCandidate = 10;
    pi1.minHRI       = 1;
    pi1.minHIO       = 18;
    pi1.minHSI       = 6;
    pi1.useVdsOnce   = 1;
    rcK892_RC2->addDaughter(pi1);

    attribute k1;
    k1.topp         = kinema::findTopp(321);
    k1.mass         = .493677;
    k1.minP         = 0;
    k1.maxP         = 200;
    k1.minPt        = 0.3;
    k1.minLRK       = .001;
    k1.maxCandidate = 10;
    k1.minHRI       = 1;
    k1.minHIO       = 18;
    k1.minHSI       = 6;
    k1.useVdsOnce   = 1;
    rcK892_RC2->addDaughter(k1);

    if (verb) rcK892_RC2->printDaughters();
  }
  rcK892_RC2->rc();
  if (verb) cout<<">>>>>>>>>>>>>>>>>>  K892_RC2 <<<<<<<<<<<<<<<<<<<"<<endl;
  return 0;
}



//
// K* selection, returns 1 upon success, 0 otherwise
//
int
selectK892()
{
  ARTE_mark( "rcX_K* 0", 0 );
  static int old_total1=0;
  static int old_total2=0;
  int ret=0;
  maximizeRTRA();
  rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1060 ), 0., 1.); // new event
  ArteTable<RTRA>::iterator k1=ArteTable<RTRA>::end();
  K892();
  ArteTable<RTRA>::iterator k2=ArteTable<RTRA>::end();
  if (rcK892_RC1->total>old_total1) { old_total1=rcK892_RC1->total; ret=1; }
  if (rcK892_RC2->total>old_total2) { old_total2=rcK892_RC2->total; ret=1; }
  if (ret) {
    rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1060 ), 1., 1.); // event selected
    // -- fill mass
    for (ArteTable<RTRA>::iterator ik=k1; ik!=k2; ++ik) {
      ArtePointer<RVER> rver=(*ik)->ever;
      float mass=rver->mass;
      rhp_hf1(DQ_ID( COMP_RECO,LVL_REC, 1061 ), mass, 1.);
    }
  }
  ARTE_mark( "rcX_K* 1", 0 );
  return ret;
}


//
//
//
void
stopK892()
{
  if (rcK892_MC1) delete rcK892_MC1;
  if (rcK892_MC2) delete rcK892_MC2;
  if (rcK892_RC1) delete rcK892_RC1;
  if (rcK892_RC2) delete rcK892_RC2;
}
