
//*****************************************************************************
// 
// Author:  Rainer Mankel
//          Humboldt University Berlin
//          mankel@ifh.de
//
// Created: 02 Mar 1999
//
// Versions history:
//
// 1.03/01 - add detailed timing (stored in EVTI) (AG)
//
// 1.03/00 - storage of neutrals added, bremsstrahlung correction foreseen.
//           Matching VDS/main tracker added.
//
// 1.02/03 - Jpsi vertex search removed in rcvert()
//
// 1.02/02 - track-based muon reco added
//
// 1.02/01 - call to RITER and VDS pattern recognition (cats,holmes)
//           added
//
// 1.02/00 - RICH reconstruction added. Reconstruction options.
//
// 1.01/00 - matching between tracks and reconstructed ECAL 
//           clusters added (using matchRsegCare). 
//    	     A first simple electron identification (rcelid.C)
//           was added & included in rcevnt. rcinit.C added
//           for module initialization.  (2-Mar-99)
//
// 1.00/00 - first version of default reconstruction, includes
//           ideal VDS reconstruction, main tracker reconstruction,
//           ideal matching and full refit.
//
// Changes:
// 25 May 2000 (AG) add ARTE_mark()
// 14 May 2000 (AG) more ARTE_mark()
// 16 Jun 2000 (AG) polishing
// 25 Jul 2000 (AG) added classification initialization
// 03 Aug 2000 (AG) added printout of classes
//
//*****************************************************************************

// ARTE includes
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "reco/recoOptions.hh"
#include "reco/eclass.hh"
#include "farm/farm.hh"

// globals
int *classSum = 0;

// externals
extern recoOptions RecoOpt;

// prototypes
int SvdInit();

//
// program code
//
extern "C" void
rcinit_()
{
  // mark
  ARTE_mark( "rcinit_ 0", 0 );
  
  // inits
  static int init = 0;
  
  if( !init ){

    // message
    arteMsg << "HERA-B reconstruction version 1.03/01";
    arteMsg.warn();
    
    // SVD
    ARTE_mark( "SvdInit", 0 );
    SvdInit();
    
    // set default activations for algorithms
    RecoOpt.SetInActiveAlg( "HOLMES" );
    RecoOpt.SetInActiveAlg( "CATS" );
    RecoOpt.SetInActiveAlg( "RIRE" );
    RecoOpt.SetInActiveAlg( "RITER" );
    RecoOpt.SetInActiveAlg( "MURECA" );
    RecoOpt.SetInActiveAlg( "MURECB" );
    RecoOpt.SetInActiveAlg( "VERTEX" );
    
    init = 1;
  }

  // printout
  cout << "CLASS: event classification bits" << endl;
  for( int bit=1; bit<=16; bit++ ){
    cout << "       " << bit   << " " << rcClassName( bit ) << endl;
  }

  // classification
  classSum = new int[16];
  for( int i=0; i<16; i++ ) classSum[i] = 0;

  // mark
  ARTE_mark( "rcinit_ 1", 0 );
}
