
// ********************************************************************
//
// Routine:      rcstop
//
// Description:  reconstruction termination
//
// Modification:
// 10 Mar 2000 (TO) added MARPLE termination
// 25 Jul 2000 (AG) added classification statistics
// 26 Jul 2000 (AG) corrected printout
// 28 Jul 2000 (AG) improved printout of CLASS
// 31 Jul 2000 (AG) introduced writing to logfile
// 02 Aug 2000 (AG) solved potential memory leack
// 04 Aug 2000 (AG) polished printout
// 06 Aug 2000 (AG) introduced stopB().
//
// ********************************************************************
 
// standard includes

#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>

// ARTE includes
#include "arte/ArteFlag.h"
#include "farm/farm.hh"
#include "marple/marple.hh"

// externals
extern int  *classSum; // class statistics
extern char *LOGFILE;  // log-file

// prototypes
extern "C" void Riter_stop_();
void stopKs();
void stopK892();
void stopPhi();
void stopD0();
void stopB();

//
// function code
//
extern "C" int
rcstop_()
{
  ARTE_mark( "rcstop_ 0", 0 );
  cout << "rcstop_" << endl;

  // RITER termination
  ARTE_mark( "Riter_stop_", 0 );
  Riter_stop_(); 
  
  // MARPLE termination
  ARTE_mark( "marplestop_", 0 );
  marplestop_();

  // VZERO2
  stopKs();
  stopK892();
  stopPhi();
  stopD0();

  // DVTX
  stopB();

  // classification statistics
  if( classSum ){
    static char strg[300];
    memset( strg, '\0', 300 );
    ostrstream cstr( strg, 300 );
    cstr << " CLASS: ";
    cstr << "tot=" << arteflag_.nevents << " ";
    for( int i=0; i<16; i++ ){
      cstr << "#" << i+1 << "=" << classSum[i] << " "; 
    }
    
    // printout
    cout << strg << endl;
    
    // write to logfile
    ARTE_log( LOGFILE, strg );
  }
  
  ARTE_mark( "rcstop_ 1", 0 );
  return 0;      
}
