
//**********************************************************************/
// Created: 08.03.99, by Ignacio Ari\~no   for the RICH reconstruction 
//         from tracks
//
// implementation of RichTrack class
//
//**********************************************************************/


#include <arte/arte.hh>

#include "rire/richreco.hh"

#include "rich/PhiLambda.h"
#include "rich/rich.h"

#include "CLHEP/Hist/HBookTuple.h"

//extern "C" float geomeff_(float *x,float *p,float *costhe,int *n,
//			  float *mina,float *maxa);
extern "C" float geomeff_(float *x,float *p,float *costhe,int *n);


RichTrack::RichTrack()
{
  track= new RcSegPoint();
  Readconstantsfromfile();
    //default constructor
}


RichTrack::RichTrack(ArtePointer<RTRA> aprtra)
{
  Readconstantsfromfile();

  //look for RPNT closer to the begin of RICH
  ArtePointer<RPNT> closerpnt;
  closerpnt->zf=0;


  for(ArteRelationIterator<RPNT> rpntit=aprtra->rpnt.begin();
      rpntit!=aprtra->rpnt.end();rpntit++)
    {
      if((*rpntit)->zf>closerpnt->zf && (*rpntit)->zf<zRichmax)
	closerpnt=*rpntit;
    }

  Hep3Vector point(closerpnt->xf,closerpnt->yf,closerpnt->zf);

  //if there is no point after magnet give momentum 0
  //so that no likelihoods are calculated (provisional)
  if(closerpnt->zf>500){
    RcPoint rcp(point,closerpnt->txf,closerpnt->tyf,closerpnt->pf,
		closerpnt->cvf,0);

    if(ArteTable<MTRA>::begin()==ArteTable<MTRA>::end()){
      //corrections for real data
      float errtx2=closerpnt->cvf[5]*4;
      if(sqrt(errtx2)<0.0008) errtx2=pow(0.0008,2);
      rcp.setcov(3,3,errtx2);
      float errty2=closerpnt->cvf[9]*4;
      if(sqrt(errty2)<0.001) errty2=pow(0.001,2);
      rcp.setcov(4,4,errty2);
      //correction for global shift of OTR
      
      float corrty=rcp.ty();
      if(rcp.x()>0) corrty-=0.002*5.*fabs(closerpnt->pf);
      rcp.setty(corrty);
    }
    track = new RcSegPoint(rcp);

    track->prolongate(ZMidRich());
  }
  else{
    RcPoint rcp(point,closerpnt->txf,closerpnt->tyf,10000001,
		closerpnt->cvf,0);
 
    track = new RcSegPoint(rcp);
    //track->prolongate(ZMidRich());
  }

  //beampipe shadow
  SetShadow();

  for(int i=0;i<7;i++)  lnlikelihoods[i]=0;

  backslope=0;

  for(int i=1;i<6;i++){
    float beta=1./sqrt(1.+pow(masshyp(i)*track->pe(),2));
    if(beta<1./Meanrefrindex) 
      thetacer[i]=0;
    else 
      thetacer[i]=acos(1./(beta*Meanrefrindex));
  }

}


RichTrack::~RichTrack()
{
  delete track;
}





void RichTrack::Readconstantsfromfile()
{
  //find vessel dimensions from ARTE tables.
  for(ArteTable<GESL>::iterator geslit=ArteTable<GESL>::begin();
      geslit!=ArteTable<GESL>::end();geslit++){
    if((*geslit)->cmp==5){
      zRichmin=(*geslit)->z1;
      zRichmax=(*geslit)->z2;
      break;
    }
  }

  if(Meanrefrindex==0){
    if(ArteTable<MTRA>::begin()!=ArteTable<MTRA>::end())
       Meanrefrindex=1.00138;
    else{
      //int run=(*ArteTable<EVHD>::begin())->run;
      //float radbeta1=getRadius(run,0);
      float radbeta1=ArtePointer<RITH>(1)->rad0;
      Meanrefrindex=1/cos(radbeta1);
    }
  }
             
  //provisional: constants set here

  maxtheta=0.065;
  //  numsigmas=3.0;
  numsigmas=Nsigmas;
  //nbins=40;
  nbins=Nbins_back;

  Sigma_spherical=0.00016;//not fully understood 
  Sigma_cromatic=0.0005;
  Sigma_cellsize=2./(1140.*sqrt(12.)); //to be multiplied by the cellsize

}




void RichTrack::SetShadow()
{
  float azi0=atan(track->y()/track->x());
  if(track->x() < 0) azi0+=M_PI;
  if(track->x() > 0 && track->y() < 0) azi0+=2.*M_PI;

  //only for tracks
  if(track->y()>0) 
    azi0+=M_PI;
  else
    azi0-=M_PI;

  float dist=sqrt(track->x()*track->x() + track->y()*track->y());
  ArtePointer<CVES> apcves(1);
  float bprad=(apcves->prad)/10;

  if(dist>bprad){
    float theta=asin(bprad/dist);
    minazi=azi0-theta;
    maxazi=azi0+theta;

  }
  else
    {
      minazi=10;
      maxazi=10;
    }

  // minazi=0;
  // maxazi=0;

}





float RichTrack::NumExpPhotons(int hyp)
{

  if(minazi==10 && maxazi==10) return 0;//inside beam pipe

  double beta=1./sqrt(1+pow(masshyp(hyp)*track->pe(),2));
  if(beta<1/Meanrefrindex) return 0;

  float theta_cer=ThetaCer(hyp);
  
  float nphotons= Nphotbeta1 * geomefi(theta_cer) * 
    (1.-1./pow(Meanrefrindex*beta,2))/(1.-1./pow(Meanrefrindex,2));

  return nphotons;  
}



float RichTrack::RelTheta(float hitphi,float hitlambda,int up=0)
{

  //corrections for spherical aberrations (up=0, down=1)
  
  float dphi= RICH_LambdaPhi->Correct()->dPhi(hitphi,hitlambda,1-up);
  float dlam=RICH_LambdaPhi->Correct()->dLambda(hitphi,hitlambda,1-up);

  //alignment corrections?
  //  float dy=(TrackLambda()-hitlambda-P_verti+0.001)/D_verti;

  return sqrt(dphi*dphi + dlam*dlam);
}




float RichTrack::BackSlope(float sigmamean,int nbins,
			    int numback,int *histoback)
{
  float binsize=maxtheta/float(nbins);

  // Calculate accessible area
  
  float thetal=0;
  float thetah=0;
  float lastthetah=0;
  float area=0;
  float theta_calc=0;

  for(int j=5;j>0;j--){
    theta_calc=ThetaCer(j);
    if(theta_calc==0) continue;
    thetal=theta_calc-numsigmas*sigmamean;
    thetah=theta_calc+numsigmas*sigmamean;
    if(lastthetah<thetal){
      area+=3.141592*(pow(thetal,2)-pow(lastthetah,2))*
	geomefi((thetal+lastthetah)/2);
    }
    lastthetah=thetah;      
  }
  if(lastthetah<maxtheta){
    area+=3.141592*(pow(maxtheta,2)-pow(lastthetah,2))*
      geomefi((maxtheta+lastthetah)/2);
  }  

  if(area==0) {
    back_alpha=1.;
    back_beta=0.;
    backslope=0.;
    return 0;
  }

  
  float alpha=1;
  float beta=0;

  if(numback==0) {
    backslope=0.5/area;
    back_alpha=1;
    back_beta=0;
    return backslope;
  }

  float lastchi=10000;
  
  backslope=float(numback)/area;
  
  float alphaprobe=0.02;
  float betaprobe=-0.1;
  //float alphastep=0.1;
  //float betastep=-5;
  
  float chi=back_chi(backslope,alpha,beta,nbins,binsize,histoback);
  int steps=0;
  while((lastchi-chi)>chi*0.01 && steps<10){
    steps++;
    lastchi=chi;
    float chialpha=back_chi(backslope,alpha+alphaprobe,beta,nbins,binsize,
			    histoback);
    float chibeta=back_chi(backslope,alpha,beta+betaprobe,nbins,binsize,
			   histoback);
    alpha+=100*(lastchi-chialpha)*alphaprobe/lastchi;
    beta+=100*(lastchi-chibeta)*betaprobe/lastchi;
    chi=back_chi(backslope,alpha,beta,nbins,binsize,histoback);
  }  

  back_alpha=alpha;
  back_beta=beta;
  return backslope;
}




float RichTrack::back_chi(float slope, float alpha, float beta, int nbins,
			   float binsize,int *histoback)
{
  float chi=0;
  for(int i=0;i<nbins;i++){
    if(histoback[i]==-5) continue;
    float th=(i+0.5)*binsize;
    float fun=backslope*binsize*2*3.141592*(alpha*th+beta*th*th+
					    beta*binsize*binsize/12.)*
      geomefi(th);
    if(histoback[i]!=0) chi+=pow(histoback[i]-fun,2)/histoback[i];
    else chi+=fun*fun;
  }
  return chi;
}



float RichTrack::geoeff(float theta)
{

  float x[3];
  float p[3];
  x[0]=track->x();
  x[1]=track->y();
  x[2]=track->z();

  p[0]=RichDir().x();
  p[1]=RichDir().y();
  p[2]=RichDir().z();
  //  int n=int(40*theta/maxtheta);
  //  if(n<20) n=20;
  int n=30+int(20*theta/maxtheta);

  float ctheta=cos(theta);


  float geoef=geomeff_(x, p, &ctheta,&n);

  float mina=minazi;
  float maxa=maxazi; 

  //float geoef=geomeff_(x, p, &ctheta,&n,&mina,&maxa);
  return geoef;
}


void RichTrack::Fillgeomefi()
{

  float maxefi=0;
  int imaxefi=0;
  for(int i=0;i<15;i+=2){
    //for(int i=0;i<15;i++){
    geomefis[i]=geoeff(maxtheta*float(i+0.5)/15);
    if(maxefi<geomefis[i]) {
      maxefi=geomefis[i];
      imaxefi=i;
    }
  }
  
  for(int i=1;i<15;i+=2){
    geomefis[i]=(geomefis[i-1]+geomefis[i+1])/2;
  }

  for(int i=0;i<15;i++){
    if(geomefis[i]<maxefi/2.5 && i>imaxefi) geomefis[i]=maxefi/2.5;
    if(geomefis[i]<maxefi/1.5 && i<imaxefi) geomefis[i]=maxefi/1.5;
  }

}
    





void RichTrack::FindLikelihoods(RichHits *richhits)
{

  //out of acceptance.
  if(fabs(1/Track()->pe())<0.0001 || (minazi==10 &&maxazi==10)){
    for(int i=1;i<7;i++){
      lnlikelihoods[i]=1./6.;
    }
    return;
  }          

  Fillgeomefi();

  //find the list of candidates for any hypothesis and the list of 
  //background photons.

  vector<Pcand*> VCand;

  //double sig_center=sqrt(pow(SigmaPhiTrack(),2)+pow(SigmaLambdaTrack(),2)); 
  //double sig2=pow(sig_center,2)+pow(Sigma_spherical,2)+pow(Sigma_cromatic,2);
  float sig2=pow(Sigma_spherical,2)+pow(Sigma_cromatic,2);

  float theta_el=ThetaCer(1);
  //Optical corrections
  //displacements at z=0 (m)
  float x0=(RichEntryPoint().x()-RichDir().x()*RichEntryPoint().z())*0.01;
  float y0=(RichEntryPoint().y()-RichDir().y()*RichEntryPoint().z())*0.01;
  RICH_LambdaPhi->Correct()->newRing(atan(RichDir().x()),atan(RichDir().y())
				     ,theta_el,x0,y0);
  float sigmamean=0;
  int numcand=0;
  int numback=0;

  float binsize=maxtheta/float(nbins);

  int *histoback=new int[nbins]; 
  for(int i=0;i<nbins;i++){
    histoback[i]=0;
  }

  float sigmaphi=SigmaPhiTrack();

  if(sigmaphi>0.004) sigmaphi=0.004;
  float sigmalambda=SigmaLambdaTrack(); 
  if(sigmalambda>0.004) sigmalambda=0.004;
 
  float tphi=TrackPhi();
  float tlambda=TrackLambda();

  float trackangle=sqrt(tphi*tphi+tlambda*tlambda);
  for(int i=0;i<richhits->numhits();i++)
    {
      float thetai=RelTheta(richhits->Phi(i),richhits->Lambda(i),
			    richhits->Up(i));
      if(thetai>maxtheta) continue;
      
      //beam pipe shadow
      int shadow=0;
      float dphi=richhits->Phi(i)-tphi;
      float dlambda=richhits->Lambda(i)-tlambda;

      float azi=atan(dlambda/dphi);
      if(dphi < 0) azi+=M_PI;
      if(dphi > 0 && dlambda <0) azi+=2*M_PI;

      //if(azi>minazi && azi<maxazi && thetai>trackangle) shadow=1;
      //if(minazi<0 && azi-2*M_PI>minazi && thetai>trackangle) shadow=1;
    
      if(azi>minazi && azi<maxazi) shadow=1;
      if(minazi<0 && azi-2*M_PI>minazi) shadow=1;
      if(shadow) continue;


      float sigmai=sqrt(sig2 + pow(sigmaphi*dphi/thetai,2)+
			pow(sigmalambda*dlambda/thetai,2)+
			pow(Sigma_cellsize*richhits->Cellsize(i),2));
      
      int ring=0;
      for(int j=1;j<6;j++){
	float theta_calc=ThetaCer(j);
	if(fabs(thetai-theta_calc)<numsigmas*sigmai && theta_calc!=0) {
	  ring=1;
	  break;
	}
      }

      sigmamean+=sigmai;
      if(ring){
	Pcand *pcand=new Pcand(thetai,sigmai);
	VCand.push_back(pcand);
	//test:
	int ibin=int(thetai*float(nbins)/maxtheta);
	histoback[ibin]++;
      }
      else{ 
	numback++;
	int ibin=int(thetai*float(nbins)/maxtheta);
	if(ibin>nbins-1) ibin=nbins-1;
	histoback[ibin]++;
      }
    }
  
  if(VCand.size()+numback>0)
    sigmamean/=float(VCand.size()+numback);
  else
    sigmamean=sqrt(sig2 + pow(Sigma_cellsize*0.9,2));
  
  for(int j=1;j<6;j++){
    float theta_calc=ThetaCer(j);
    if(theta_calc==0) continue;
    for(int i=0;i<nbins;i++){
      if(fabs(float(i)*maxtheta/float(nbins)-theta_calc)<numsigmas*sigmamean ||
	 fabs(float(i+1)*maxtheta/float(nbins)-theta_calc)<numsigmas*sigmamean)
	{
	  histoback[i]=-5; //could be assigned to a ring.
	  //break;
	}
    }
  }
  float bslope=BackSlope(sigmamean,nbins,numback,histoback);

  //Calculate likelihoods for each hypothesis

  int imin=5;
  int imax=5;
  
  double lnlike=0;
  double likeback=0;

  for(int j=5;j>0;j--){
    float thetai=ThetaCer(j);
    if(thetai==0) {
      imin=j-1;
      imax=j-1;
      continue;
    }
    if(j>1) {
      float thetanext=ThetaCer(j-1);
      if((thetai+numsigmas*sigmamean)>(thetanext-numsigmas*sigmamean)){
	imin=j-1;
      }else{
	likeback=0;
	for(int ihyp=imin;ihyp<=imax;ihyp++){
	  lnlikelihoods[ihyp]+=like(ihyp,imin,imax,sigmamean,&likeback,VCand);
	}
	for(int i=1;i<6;i++){
	  if(i<imin || i>imax) lnlikelihoods[i]+=likeback;
	}
	lnlikelihoods[6]+=likeback;
	imin=j-1;
	imax=j-1;
      }
    }else{	
      likeback=0;
      for(int ihyp=1;ihyp<=imax;ihyp++){
	lnlikelihoods[ihyp]+=like(ihyp,1,imax,sigmamean,&likeback,VCand);
      }
      for(int i=1;i<6;i++){
	if(i>imax) lnlikelihoods[i]+=likeback;
      }
      lnlikelihoods[6]+=likeback;
    }
  }
//likelihood normalisation

  float hlik=0;
  for(int i=1;i<7;i++){
    if(hlik<lnlikelihoods[i]) hlik=lnlikelihoods[i];
  }
  float sum=0;
  for(int i=1;i<7;i++){
    sum+=exp(lnlikelihoods[i]-hlik);
  }
  for(int i=1;i<7;i++){
    if(exp(lnlikelihoods[i]-hlik)/sum > 1.E-50)
      lnlikelihoods[i]=exp(lnlikelihoods[i]-hlik)/sum;
    else
      lnlikelihoods[i]=1.E-50;
  }

  //delete VCand
  for(vector<Pcand*>::iterator vcit=VCand.begin();
      vcit!=VCand.end();vcit++)
    {
      delete *vcit;
    }
  VCand.erase(VCand.begin(),VCand.end());

  delete[] histoback;

}
  


double RichTrack::like(int ihyp, int imin, int imax,float sigmamean,
		       double *likeback,vector<Pcand*> VCand)
{
  float nexpp=NumExpPhotons(ihyp);
  float theta_calc=ThetaCer(ihyp);
  float theta_min=ThetaCer(imax);
  float theta_max=ThetaCer(imin);
  
  float nback=2.*3.141592*backslope*
    ((back_alpha/2.0)*(pow(theta_max+numsigmas*sigmamean,2)-
		      pow(theta_min-numsigmas*sigmamean,2))
    +(back_beta/3.0)*(pow(theta_max+numsigmas*sigmamean,3)-
		      pow(theta_min-numsigmas*sigmamean,3)))*
    geomefi((theta_max+theta_min)/2);
  
  //test
  if(nback<0.5) nback=0.5;
  
  int ncand=0;
  for(unsigned int i=0;i<VCand.size();i++)
    {
      float thetai=VCand[i]->Theta();
      float sigmai=VCand[i]->Sigma();
      if(thetai>theta_min-numsigmas*sigmai && 
	 thetai < theta_max+numsigmas*sigmai) ncand++;
    }

  double like=0;
  if(ncand==0){
    //if(imin<=ihyp && ihyp<=imax) 
    like=-(nexpp+nback);
    *likeback=-nback;

    return like;
  }
  float fracs=0;
  float fracb=1;
  *likeback=ncand*log(nback)-(nback)-(ncand*log(ncand)-ncand);

  like=ncand*log(nexpp+nback)-(nexpp+nback)-(ncand*log(ncand)-ncand);
  fracs=nexpp/(nexpp+nback);
  fracb=nback/(nexpp+nback);

  for(unsigned int i=0;i<VCand.size();i++)
    {
      float thetai=VCand[i]->Theta();
      float sigmai=VCand[i]->Sigma();
      
      if(thetai < theta_min-numsigmas*sigmai || 
	 thetai > theta_max+numsigmas*sigmai) continue;

      float normback=(pow(theta_max+numsigmas*sigmai,2)-
		      pow(theta_min-numsigmas*sigmai,2))*back_alpha/2.
	+(pow(theta_max+numsigmas*sigmai,3)-
	  pow(theta_min-numsigmas*sigmai,3))*back_beta/3.;
      
      like+=log(fracs*exp(-0.5*pow((thetai-theta_calc)/sigmai,2))/
		(2.48156*sigmai)+
		fracb*(thetai*back_alpha+thetai*thetai*back_beta)/normback);
      //(sqrt(2.*M_PI)*sigmai*0.99)=2.48156;
      *likeback+=log((thetai*back_alpha+thetai*thetai*back_beta)/normback);
    }
  return like;
}
















