//-----------------------------------------------------------------------//
//
//  Attributes for stand-alone ring reconstruction
//  Author: Denis Dujmic
//  Created: May-00
//
//  Modifications:
//    29 May, 2000 (DD): RingSearchAttrib::use_underdogs=false
//    May 29, 2000 (DD): RingSearchAttrib::finer_search_step added 
//    Jun 06, 2000 (DD): kuip commands added
//    Jun 08, 2000 (DD): minHsi,minHin,... added
//    Jun 09, 2000 (DD): new kuip commands
//    Jun 13, 2000 (DD): default fill rtra = false
//    Jul 01, 2000 (DD): Finer search step = false
//    Jul 06, 2000 (DD): use normalized prob. by default  
//    Jul 28, 2000 (DD): fill rtra by default  
//                      
//-----------------------------------------------------------------------//
#include "rise/RingSearchAttrib.h"

bool  RingSearchTrack::exist=false;
float RingSearchTrack::bx=0;
float RingSearchTrack::by=0;
float RingSearchTrack::phi=0;
float RingSearchTrack::lambda=0;
float RingSearchTrack::ip=0;
ArtePointer<RTRA> RingSearchTrack::rtra=0;
ArtePointer<RSEG> RingSearchTrack::rseg=0;
float RingSearchTrack::max_phi_residual=.005;
float RingSearchTrack::max_lambda_residual=.005;
float RingSearchTrack::max_rad_residual=.0075;
float RingSearchTrack::aa=.03*.03;
float RingSearchTrack::bb=.0025*.0025;
float RingSearchTrack::sigma2_rr  = 1.5e-5 * 1.5e-5;
float RingSearchTrack::min_rpnt_z=550;
float RingSearchTrack::max_rpnt_z=1350;

int   RingSearchAttrib::type=RingSearchType::nobg;
bool  RingSearchAttrib::reuse=true;
bool  RingSearchAttrib::bookrseg=true;
bool  RingSearchAttrib::bookhitb=true;
bool  RingSearchAttrib::fillrtra=true;
float RingSearchAttrib::SNcut=3;      
float RingSearchAttrib::Scut=10.;
int   RingSearchAttrib::seedOccupancyCut=20000;
int   RingSearchAttrib::occupancyCut=20000;
int   RingSearchAttrib::peakCut=5;        
float RingSearchAttrib::minSearchRadius=.03;
float RingSearchAttrib::maxSearchRadius=.0526;
float RingSearchAttrib::searchStep=-1;
int   RingSearchAttrib::peakThreshold=5;
bool  RingSearchAttrib::renorm=true;
bool  RingSearchAttrib::id=true;
bool  RingSearchAttrib::use_simple_R2P2=false;
bool  RingSearchAttrib::use_seed_phi=false;
bool  RingSearchAttrib::use_seed_lambda=false;
int   RingSearchAttrib::choose_best_ring=1;
int   RingSearchAttrib::seed_type=RingSearchType::rtra_seed;
bool  RingSearchAttrib::use_underdogs=false;
bool  RingSearchAttrib::finer_search_step=false;
int   RingSearchAttrib::rlevel=1;
int   RingSearchAttrib::minHsi=0;
int   RingSearchAttrib::minHin=0;
int   RingSearchAttrib::minHou=0;
int   RingSearchAttrib::minHio=0;
int   RingSearchAttrib::minHca=0;
int   RingSearchAttrib::minHmu=0;
int   RingSearchAttrib::overwriteOccupancyCut=-10000;
float RingSearchAttrib::overwriteMinSearchRadius=-10000;
float RingSearchAttrib::overwriteMaxSearchRadius=-10000;
float RingSearchAttrib::overwriteSearchStep=-10000;
float RingSearchAttrib::overwritePeakThreshold=-10000;
int   RingSearchAttrib::fitLevel=1;

#ifdef USE_ARTE

extern "C" {
  void kugetr_(float &par);
  void kugeti_(int   &par);

  void risesetres_() {
    float max_dphi, max_dlam, max_drad;
    kugetr_(max_dphi);
    kugetr_(max_dlam);
    kugetr_(max_drad);
    RingSearchTrack::max_phi_residual=max_dphi;
    RingSearchTrack::max_lambda_residual=max_dlam;
    RingSearchTrack::max_rad_residual=max_drad;
    cout<<" $ RISE: seed residuals dphi/dlambda/dradius: "
	<<RingSearchTrack::max_phi_residual<<"/"
	<<RingSearchTrack::max_lambda_residual<<"/"
	<<RingSearchTrack::max_rad_residual<<endl;
  }

  void risesetdpp_() {
    float a,b;
    kugetr_(a);
    kugetr_(b);
    RingSearchTrack::aa=a*a;
    RingSearchTrack::bb=b*b;
    cout<<" $ RISE: dp/p parameters A^2/B^2: "
	<<RingSearchTrack::aa<<"/"
	<<RingSearchTrack::bb<<endl;
  }

  void risebook_() {
    int hitb, rseg, rtra;
    kugeti_(hitb);
    kugeti_(rseg);
    kugeti_(rtra);
    if (hitb>=0) RingSearchAttrib::bookhitb=hitb;
    if (rseg>=0) RingSearchAttrib::bookrseg=rseg;
    if (rtra>=0) RingSearchAttrib::fillrtra=rtra;
    cout<<" $ RISE table booking: ";
    if (hitb>=0) cout<<" hitb="<<RingSearchAttrib::bookhitb;
    if (rseg>=0) cout<<" rseg="<<RingSearchAttrib::bookrseg;
    if (rtra>=0) cout<<" rtra="<<RingSearchAttrib::fillrtra;
    cout<<endl;
  }

  void riselike_() {
    int r2p2, renorm;
    kugeti_(r2p2);
    kugeti_(renorm);
    if (r2p2>=0)   RingSearchAttrib::use_simple_R2P2=r2p2;
    if (renorm>=0) RingSearchAttrib::renorm=renorm;
    cout<<" $ RISE likelihood output: r2p2="<<RingSearchAttrib::use_simple_R2P2
	<<" renormalization="<<RingSearchAttrib::renorm<<endl;
  }


  void risehset_() {
    int hsi, hin, hou, hio, hca, hmu;
    kugeti_(hsi);
    kugeti_(hin);
    kugeti_(hou);
    kugeti_(hio);
    kugeti_(hca);
    kugeti_(hmu);
    RingSearchAttrib::minHsi=hsi;
    RingSearchAttrib::minHin=hin;
    RingSearchAttrib::minHou=hou;
    RingSearchAttrib::minHio=hio;
    RingSearchAttrib::minHca=hca;
    RingSearchAttrib::minHmu=hmu;
    cout<<" $ RISE: minimum hits for RTRA search hsi="<<RingSearchAttrib::minHsi
	<<" hin="<<RingSearchAttrib::minHin<<" hou="<<RingSearchAttrib::minHou
	<<" hin+hou="<<RingSearchAttrib::minHio
	<<" hca="<<RingSearchAttrib::minHca
	<<" hmu="<<RingSearchAttrib::minHmu<<endl;
  }



  void risesearch_() {
    
    int   occup;
    float min, max, step, thre, sn, s;
    int   rlevel;
    int   fit, choose, seedoccup, finestep;
    kugeti_(rlevel);
    kugeti_(occup);
    kugetr_(min);
    kugetr_(max);
    kugetr_(step);
    kugetr_(thre);
    kugeti_(fit);
    kugeti_(choose);
    kugetr_(sn);
    kugetr_(s);
    kugeti_(seedoccup);
    kugeti_(finestep);
    if (rlevel>=0) RingSearchAttrib::rlevel=rlevel;
    RingSearchAttrib::overwriteOccupancyCut=occup;
    RingSearchAttrib::overwriteMinSearchRadius=min;
    RingSearchAttrib::overwriteMaxSearchRadius=max;
    RingSearchAttrib::overwriteSearchStep=step;
    RingSearchAttrib::overwritePeakThreshold=thre;

    RingSearchAttrib::fitLevel=fit;
    if (fit>0) switch (fit) {
    case 1:
      RingSearchAttrib::type=RingSearchType::nobg; break;
    case 2:
      RingSearchAttrib::type=RingSearchType::lkbg; break;
    case 3:
      RingSearchAttrib::type=RingSearchType::nobg; break;
    }

    RingSearchAttrib::choose_best_ring=choose;

    if (sn>0) RingSearchAttrib::SNcut=sn;
    if (s>0)  RingSearchAttrib::Scut=s;
 
    if (seedoccup>0) RingSearchAttrib::seedOccupancyCut=seedoccup;

    if (finestep>=0) RingSearchAttrib::finer_search_step=finestep;

    cout<<" $ RISE search: level="<<RingSearchAttrib::rlevel;
    cout<<" fit="<<RingSearchAttrib::fitLevel;
    cout<<" type="<<RingSearchAttrib::type;
    if (sn>0) cout<<" SN="<<RingSearchAttrib::SNcut;
    if (s>0)  cout<<" S="<<RingSearchAttrib::Scut;
    cout<<" choose="<<RingSearchAttrib::choose_best_ring<<endl;
  }


}

#endif
