*-- Author :    T. Oest   24/10/96

      SUBROUTINE V0HITD(IGEDE, DP1, DP2, XOUT, YOUT, ZOUT, MUSC, IRET)

************************************************************************
*
* Description: point where the track hits a detector plane
*              tracks are supposed to be straight tracks
*
* Called from:
*
* External references:
*
* Argument in/out description
* IGEDE    in     wall number
* DP1      in     first  point (z,x,y,tx,ty,rho)
* DP2      in     second point (z,x,y,tx,ty,rho)
* XOUT     out    x position where track hits detector plane
* YOUT     out    y position where track hits detector plane
* ZOUT     out    z position where track hits detector plane
* MUSC     out    multiple scattering angle for a 1 GeV track
* IRET     out    =  1  trajectory hits pipe
*                 =  0  trajectory doesn't hit detector plane
*                 = -1  error
*
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/GEDE.fi"

      INTEGER          IGEDE, IRET
      DOUBLE PRECISION DP1(6), DP2(6), XOUT, YOUT, ZOUT, MUSC


      INTEGER          I
      REAL*4           P1(6), P2(6)
      REAL*4           X, Y, Z, ZWALL, XROT, YROT, ZROT, FAC
      REAL*4           ROT(3,3), RW(3), ZWMAX, ZWMIN

#include "arte/Arte.ff"
#include "auto/GEDE.ff"

*-----------------------------------------------------------------------
* initialization
*-----------------------------------------------------------------------
      IRET = 0
      IF (IGEDE.LE.0 .OR. IGEDE.GT.NGEDE()) THEN
       IRET = -1
       GOTO 9000
      ENDIF

      DO I=1,6
       P1(I) = SNGL(DP1(I))
       P2(I) = SNGL(DP2(I))
      ENDDO

      ZWMIN = FGEDE_ZMIN(IGEDE)
      ZWMAX = FGEDE_ZMAX(IGEDE)
      ZWALL = 0.5*(FGEDE_ZMIN(IGEDE)+FGEDE_ZMAX(IGEDE))

C      IF (  P1(1).EQ.P2(1) .OR.
C     +     (P1(1).GE.ZWALL .AND. P2(1).GE.ZWALL) .OR.
C     +     (P1(1).LE.ZWALL .AND. P2(1).LE.ZWALL)) THEN
      IF (  P1(1).EQ.P2(1) .OR.
     +     (P1(1).GT.ZWMAX .AND. P2(1).GT.ZWMAX) .OR.
     +     (P1(1).LT.ZWMIN .AND. P2(1).LT.ZWMIN)) THEN
       IRET = 0
       GOTO 9000
      ENDIF
      IF (ZWALL.LT.P1(1).AND.ZWALL.LT.P2(1)) ZWALL = MIN(P1(1),P2(1))
      IF (ZWALL.GT.P1(1).AND.ZWALL.GT.P2(1)) ZWALL = MAX(P1(1),P2(1))
*-----------------------------------------------------------------------
* track point for z=zwall
*-----------------------------------------------------------------------
      FAC = (ZWALL-P1(1))/(P2(1)-P1(1))
      X = P1(2) + FAC * (P2(2)-P1(2))
      Y = P1(3) + FAC * (P2(3)-P1(3))
*-----------------------------------------------------------------------
* rotate the point
*-----------------------------------------------------------------------
      XROT = X * FGEDE_ROT(1,IGEDE) - Y * FGEDE_ROT(2,IGEDE)
      YROT = X * FGEDE_ROT(2,IGEDE) + Y * FGEDE_ROT(1,IGEDE)

      IF (XROT.GE.FGEDE_XMIN(IGEDE) .AND. XROT.LE.FGEDE_XMAX(IGEDE).AND.
     +    YROT.GE.FGEDE_YMIN(IGEDE) .AND. YROT.LE.FGEDE_YMAX(IGEDE))THEN
       IRET = 1
       XOUT = DBLE(X)
       YOUT = DBLE(Y)
       ZOUT = DBLE(ZWALL)
       MUSC = DBLE( FGEDE_MS(IGEDE) * 
     +        SQRT(((P2(1)-P1(1))**2+(P2(2)-P1(2))**2+(P2(3)-P1(3))**2)/
     +              (P2(1)-P1(1))**2))
c       print*,' igede ',igede,FGEDE_ZMIN(IGEDE),FGEDE_ZMAX(IGEDE)
c       print*,FGEDE_XMIN(IGEDE),FGEDE_XMAX(IGEDE),
c     +                  FGEDE_YMIN(IGEDE),FGEDE_YMAX(IGEDE)
      ELSE
       IRET = 0
      ENDIF
*-----------------------------------------------------------------------
* print information
*-----------------------------------------------------------------------
c      print*,' ********** v0sidW test ******************', iret
c      print*,' X,Y,Z ',X,Y,ZWALL
*-----------------------------------------------------------------------
* the end
*-----------------------------------------------------------------------
 9000 END
