// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   21. Aug 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: relation container class
//
// changes:
// 21 Aug 2000 (TG) creation
//
// *****************************************************************************
#include <iostream>
#include "vt++/VtVertex.hh"
#include "vt++/VtTrack.hh"
#include "vt++/VtRelation.hh"
#include "vt++/VtRelationList.hh"

namespace VERTEX {


  //=============================================================================
  // createRelation
  //=============================================================================
  bool createRelation(Track& t, Vertex& v) {
    //    cout << " Create Relation called!! " << &t << " " << &v << endl;
    Relation* rel = new Relation(t,v);
    t.push__back(rel);
    v.push__back(rel);
    return true;
  }

  //=============================================================================
  // Constructors
  //=============================================================================
  RelationList::RelationList() {}

  RelationList::~RelationList() {
    if(rellist.empty()) return;

    if(static_cast<RelationList*>(&rellist.back()->vertex) == this) {
      // Dtor called from ~Vertex
      //      cout << "delete in vertex! " << this << endl;
      for(Relation_cit it=rellist.begin(); it!=rellist.end(); ++it) {
	Relation* rel = *it;
	//  	cout << " Delete Relation " << rel 
	//  	     << " " << rel->track << endl;
	rel->track.remove(rel);  // unbook relation
	delete rel;              // destroy relation
      }
    } else {
      // Dtor called from ~Track
      //      cout << "delete in track! " << this << endl;
      for(Relation_cit it=rellist.begin(); it!=rellist.end(); ++it) {
	Relation* rel = *it;
	//  	cout << " Delete Relation " << rel 
	//  	     << " " << rel->vertex << endl;
	rel->vertex.remove(rel); // unbook relation
	delete rel;              // destroy relation
      }
    }
  }


  //=============================================================================
  // operator =
  //=============================================================================
  const RelationList& RelationList::operator=(const RelationList& rhs) {
    if( this == &rhs ) { return *this; }
    
    rellist = rhs.rellist;
    return *this;
  }

  //==============================================================================
  // print
  //==============================================================================
  void RelationList::print(std::ostream& os) const {
    for(Relation_cit it=rellist.begin(); it!=rellist.end(); ++it)
      cout << **it << " ";
    return;
  }

  //=============================================================================
  // operator[] easy but slow access to Relation members
  //=============================================================================
  Relation *const RelationList::operator[](unsigned int i) const {
    if(i>rellist.size()) return 0;

    Relation_cit it=rellist.begin();
    unsigned int j=0;
    while(j++<i) ++it;
    //    cout << "pos: " << j << endl;
    return *it;
  }

  //==============================================================================
  //
  // class RelationIterator member functions
  //
  //==============================================================================
  RelationIterator::RelationIterator() {}


  //==============================================================================
  // operator++
  //==============================================================================
  RelationIterator::self& RelationIterator::operator++() {
    ++iter;
    return *this;
  }

  //==============================================================================
  // operator--
  //==============================================================================
  RelationIterator::self& RelationIterator::operator--() {
    --iter;
    return *this;
  }

  //==============================================================================
  // operator*
  //==============================================================================
  Relation& RelationIterator::operator*() const { 
    return **iter;
  }

  //==============================================================================
  // operator->
  //==============================================================================
  Relation* RelationIterator::operator->() const { 
    return *iter;
  }

  //==============================================================================
  //
  // class ConstRelationIterator member functions
  //
  //==============================================================================
  ConstRelationIterator::ConstRelationIterator() {}


  //==============================================================================
  // operator++
  //==============================================================================
  ConstRelationIterator::self& ConstRelationIterator::operator++() {
    ++iter;
    return *this;
  }

  //==============================================================================
  // operator--
  //==============================================================================
  ConstRelationIterator::self& ConstRelationIterator::operator--() {
    --iter;
    return *this;
  }

  //==============================================================================
  // operator*
  //==============================================================================
  const Relation& ConstRelationIterator::operator*() const { 
    return **iter;
  }

  //==============================================================================
  // operator->
  //==============================================================================
  const Relation* const ConstRelationIterator::operator->() const { 
    return *iter;
  }
} // end of namespace VERTEX
