#ifndef __VTDISTANCE
#define __VTDISTANCE
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   21. Aug 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: an (incomplete) collection of distance functions
//
// changes:
// 21 Aug 2000 (TG) creation
//
// *****************************************************************************

template <class T> class ArtePointer;
struct GTAR;
struct RVER;
struct MVER;

namespace VERTEX {
  class Track;
  class Vertex;

  /** @name Distance functions */
  /// spatial distance betw. RVER \& MVER
  double distance(const ArtePointer<RVER>& rver , const ArtePointer<MVER>& mver);
  /// $\chi^2$ distance track - track
  double distance(Track& t1, Track& t2);
  /// spatial distance track - vertex
  double distance(const Track& t, const Vertex& v);
  /// spatial distance track - vertex
  inline double distance(const Vertex& v, const Track& t) { return distance(t,v); }
  /// $\chi^2$ distance between two vertices
  double distance(const Vertex& v1, const Vertex& v2);
  /// $\chi2$ distance vertex - space point
  double distance(const Vertex& v, double x, double y, double z);
  /// spatial distance of track to wire
  double distance(const Track& t, const ArtePointer<GTAR>& gtar);
  /// spatial distance between RVER and GTAR
  double distance(const ArtePointer<RVER>& rver, const ArtePointer<GTAR>& gtar);

} // end of namespace VERTEX
#endif
