#ifndef __VTRELATION_HH
#define __VTRELATION_HH
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   21. Aug 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Relation class
//
// changes:
// 21 Aug 2000 (TG) creation
//
// *****************************************************************************

#include <iosfwd>

namespace VERTEX {

  class Track;
  class Vertex;
  class Kalman;
  class Relation;

  /** Class for Track-Vertex-Kalman object relations.
      @memo for internal use only
  */
  //============================================================================
  // Class Relation - class for Vertex-Track-Kalman relations
  //============================================================================
  class Relation {
  public:
    /** @name Constructors */
    ///
    Relation(Track&  ttrack, Vertex& vvertex);
    ///
    ~Relation();

    /** @name Expert functions */
    ///
    void print( std::ostream& os ) const;

    VERTEX::Track&   track;  // reference to track
    VERTEX::Vertex&  vertex; // reference to vertex 
    VERTEX::Kalman&  kalman; // reference to Kalman-object
  };

  //==============================================================================
  // operator <<
  //==============================================================================
  inline std::ostream& operator<< ( std::ostream& os, const Relation& r ) {
    r.print(os);
    return os;
  }

} // end of namespace VERTEX
#endif
