#ifndef __VTVECTOR_HH
#define __VTVECTOR_HH
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   21. Aug 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: vector class
//
// changes:
// 21 Aug 2000 (TG) creation
//
// *****************************************************************************

#include <iosfwd>

namespace MATRIX {

  /** Vector class
   */
  //============================================================================
  // Class VtVector
  //============================================================================
  class VtVector {
  public:
    ///
    VtVector(const unsigned int size);
    ///
    VtVector(double v1, double v2);
    ///
    VtVector(double v1, double v2, double v3);
    ///
    VtVector(double v1, double v2, double v3, double v4);
    ///
    VtVector(double v1, double v2, double v3, double v4, double v5);
    ///
    VtVector(double v1, double v2, double v3, double v4, double v5,
	     double v6);
    ///
    VtVector(const VtVector& rhs);
    ///
    ~VtVector();

    ///
    void print(std::ostream& os) const;

    ///
    inline double  operator()(unsigned int row) const { return *(v + row); }
    ///
    inline double  operator[](unsigned int row) const { return *(v + row); }
    ///
    inline double& operator[](const unsigned int row) { return *(v + row); }

    ///
    const VtVector& operator= (const VtVector& rhs);
    ///
    const VtVector& operator+=(const double rhs);
    ///
    const VtVector& operator-=(const double rhs);
    ///
    const VtVector& operator*=(const double rhs);
    ///
    const VtVector& operator/=(const double rhs);
    ///
    const VtVector& operator+=(const VtVector& rhs);
    ///
    const VtVector& operator-=(const VtVector& rhs);
    ///
    const VtVector  operator+ (const VtVector& rhs) const;
    ///
    const VtVector  operator- (const VtVector& rhs) const;
    ///
    const double    operator* (const VtVector& rhs) const;

    ///
    inline unsigned int size() const { return v_size; }
    ///
    inline unsigned int nrow() const { return v_size; }

  private:
    unsigned int v_size;  // no of rows
    double*      v;       // data array
  };

  //==============================================================================
  // operator <<
  //==============================================================================
  inline std::ostream& operator<< ( std::ostream& os, const VtVector& t ) {
    t.print(os);
    return os;
  }

} // namespace MATRIX
#endif
