*CMZ :          20/05/96  12.58.34  by  Unknown
*-- Author :   T.Lohse
c
c------------------------------------------------------------
c
c       subroutine:   VTCHAI(ivp,ivs,par,cov,chi2c,flag)
c
c       input:    ivp   =  primary vertex index       Int
c                 ivs   =  secondary vertex index     Int
c
c       output:   par(6)          = fit parameters    R*8
c                      1          = decay length
c                      1..3       = secondary vertex (x,y,z)
c                      4,5        = slope_x, slope_y
c                      6          = momentum (1)
c                 cov             = cov(par)          R*8
c                 chi2c       =  chi**2 of fit        R*8
c                                ndf = 2
c                 flag        < 0 if fail             Int
c
c       action:   Fit parameters of particle decaying at
c                 secondary vertex. The vertex fits
c                 are also re-iterated.
c
c------------------------------------------------------------
c
      subroutine  VTCHAI ( ivp, ivs, par, cov, chi2c, flag )

      implicit none

#include "vt/vtdat.fi"

      integer ivp, ivs
      integer i3
      parameter (i3=3*ntrk+3)
      double precision 
     1  par(7), cov(7,7)
      double precision 
     1  chi2c
      integer flag

      integer flag1,ifail
      integer n, np, iter, i, j, k, l, nn, mm, l1, l2
      integer icp, ics, it
      integer momp, moms, n3

      double precision 
     1  parp(i3), covp(i3,i3), pars(i3), covs(i3,i3)
      double precision 
     1  Valp(i3,i3), Vbet(6,6), Vbetc(7,7)
      double precision 
     1  M(3,7), DbDa(7,i3), Cp(3,3)
      double precision 
     1  CM(3,7), MCM(7,7), Mdb(3)
      double precision 
     1  bet(7), betc(7), bet0(7), alp0(i3)
      double precision 
     1  delt0(3), delbet(7), dMdb(3), Adab(6), VAdab(6)
      double precision 
     1  AVAdab(i3)
      double precision 
     1  xn(ntrk), yn(ntrk), zn(ntrk), xnorm(ntrk)
      double precision 
     1  ch2old, sum, fact, chi2m
      double precision 
     1  ps(4)

      flag = -1
      if (ivp.gt.nvtmax)  goto 998
      if ( .not. vtset(ivp) )   goto 998
      if (ivs.gt.nvtmax)  goto 998
      if ( .not. vtset(ivs) )   goto 998
      icp = nconst(ivp)
      if (icp.lt.0.or.icp.gt.ncomax)   goto 998
      ics = nconst(ivs)
      if (ics.lt.0.or.ics.gt.ncomax)   goto 998
      momp = 0
      moms = 1
      if ( icp .gt. 0 ) momp = 1
      if ( fit(ivp) .eq. 2 ) momp = 1

      n  = ntr(ivs)
      np = ntr(ivp)
      n3 = 3*n+3

      if ( icp .eq. 0 ) then
        if ( fit(ivp) .eq. 0 ) then
          call VTFIT(ivp,momp,flag1)
          if (flag1.lt.0)  goto 998
        endif
        do  i = 1, 3
          parp(i) = xvs(i,ivp)
        enddo
      else
        fitm(ivp) = 0
        call VTMASS( ivp, parp, covp, chi2m, flag1 )
        if (flag1.lt.0)  goto 998
      endif

      if ( ics .eq. 0 ) then
        if ( fit(ivs) .eq. 0 ) then
          call VTFIT(ivs,moms,flag1)
          if (flag1.lt.0)  goto 998
        endif
        do  i = 1, 3
          pars(i) = xvs(i,ivs)
        enddo
        do  k = 1, n
          nn = 3*k
          do  j = 1, 3
            pars(nn+j) = qvs(j,ivs,k)
          enddo
        enddo
      else
        fitm(ivs) = 0
        call VTMASS( ivs, pars, covs, chi2m, flag1 )
        if (flag1.lt.0)  goto 998
      endif

c
c---> initial fit vector from secondary vertex (alp0)
c

      do  i = 1, n3
        alp0(i) = pars(i)
      enddo

c
c---> solve for secondary vertex constrained to primary vertex
c

      iter = 0
      ch2old = -100.

  100 continue

      iter = iter + 1
      if ( iter .gt. 100 ) then
        flag = -2
        goto 999
      endif

c
c--->  re-iterate the vertex fits
c
      if (iter.gt.1) then
c--->   primary
        fact = 1./sqrt(1.d0+betc(5)**2+betc(6)**2)
        xvs(1,ivp) = betc(2)-betc(1)*betc(5)*fact
        xvs(2,ivp) = betc(3)-betc(1)*betc(6)*fact
        xvs(3,ivp) = betc(4)-betc(1)*fact
        call VTSMOQ(ivp)
        if (icp.eq.0) then
          call VTFILT(ivp,momp,flag1)
          if (flag1.lt.0) goto 998
          call VTSMO(ivp)
        endif
        do  i = 1, 3
          parp(i) = xvs(i,ivp)
        enddo
        do  k = 1, np
          nn = 3*k
          do  i = 1, 3
            parp(nn+i) = qvs(i,ivp,k)
          enddo
        enddo
        if (icp.gt.0) then
          call VTMASS(ivp, parp, covp, chi2m, flag1)
          if (flag1.lt.0) goto 998
        endif
c--->   secondary
        xvs(1,ivs) = betc(2)
        xvs(2,ivs) = betc(3)
        xvs(3,ivs) = betc(4)
        call VTSMOQ(ivs)
        if (ics.eq.0) then
          call VTFILT(ivs,moms,flag1)
          if (flag1.lt.0) goto 998
          call VTSMO(ivs)
        endif
        do  i = 1, 3
          pars(i) = xvs(i,ivs)
        enddo
        do  k = 1, n
          nn = 3*k
          do  i = 1, 3
            pars(nn+i) = qvs(i,ivs,k)
          enddo
        enddo
        if (ics.gt.0) then
          call VTMASS(ivs, pars, covs, chi2m, flag1)
          if (flag1.lt.0) goto 998
        endif
      endif

c
c---> fill fit vector from vertex fits (bet)
c
      bet(1) = sqrt( (pars(1)-parp(1))**2
     +             + (pars(2)-parp(2))**2
     +             + (pars(3)-parp(3))**2 )
      do  k = 1, 3
        bet(k+1) = pars(k)
      enddo

      do  k = 1, n
        it = ktr(ivs,k)
        fact = 1./sqrt(1.+pars(3*k+1)**2+pars(3*k+2)**2)
        xnorm(k) = fact
        xn(k) = pars(3*k+1)*fact
        yn(k) = pars(3*k+2)*fact
        zn(k) = fact
      enddo

      ps(1) = 0.d0
      ps(2) = 0.d0
      ps(3) = 0.d0
      do  k = 1, n
        nn = 3*k+3
        ps(1) = ps(1) + xn(k)*pars(nn)
        ps(2) = ps(2) + yn(k)*pars(nn)
        ps(3) = ps(3) + zn(k)*pars(nn)
      enddo
      ps(4) = sqrt(ps(1)**2+ps(2)**2+ps(3)**2)
      fact  = 1./ps(4)
      ps(1) = ps(1)*fact
      ps(2) = ps(2)*fact
      ps(3) = ps(3)*fact

      bet(5) = ps(1) / ps(3)
      bet(6) = ps(2) / ps(3)
      bet(7) = ps(4)

      fact = ps(1)*(pars(1)-parp(1)) +
     +       ps(2)*(pars(2)-parp(2)) +
     +       ps(3)*(pars(3)-parp(3))
      if (fact.lt.0.)  bet(1) = - bet(1)

c
c---> fill covariance matrix of primary vertex
c
      if ( icp .eq. 0 ) then
        do  i = 1, 3
          do  j = 1, 3
            Cp(i,j) = CS(i,j,ivp)
          enddo
        enddo
      else
        do  i = 1, 3
          do  j = 1, 3
            Cp(i,j) = covp(i,j)
          enddo
        enddo
      endif
      call DSINV(3,Cp,3,ifail)
      if ( ifail.eq.-1 )   goto 998
c
c---> fill general covariance matrix of secondary vertex
c
      if ( ics .eq. 0 ) then
        do  i = 1, 3
          do  j = 1, 3
            Valp(i,j) = CS(i,j,ivs)
          enddo
        enddo
        do  k = 1, n
          nn  = 3*k
          do  i = 1, 3
            do  j = 1, 3
              Valp(nn+i,nn+j) = DS(i,j,ivs,k)
              Valp(nn+j,nn+i) = DS(i,j,ivs,k)
              Valp(nn+i,j)    = ES(i,j,ivs,k)
              Valp(j,nn+i)    = ES(i,j,ivs,k)
            enddo
          enddo
        enddo
        do  k = 1, n-1
          nn  = 3*k
          do  l = k+1, n
            mm  = 3*l
            do l1 = 1, 3
              do l2 = 1, 3
                sum = 0.d0
                do  i = 1, 3
                  sum = sum - ES(l1,i,ivs,k)*F(l2,i,ivs,l)
                enddo
                Valp(nn+l1,mm+l2) = sum
                Valp(mm+l2,nn+l1) = sum
              enddo
            enddo
          enddo
        enddo
      else
        do  i = 1, n3
          do  j = 1, n3
            Valp(i,j) = covs(i,j)
          enddo
        enddo
      endif

c
c--->  fill guess of fit parameters (bet0)
c
      if (iter.eq.1) then
        do  i = 1, 7
          bet0(i) = bet(i)
        enddo
      else
        do  k = 1, n
          it = ktr(ivs,k)
          fact = 1./sqrt(1.+alp0(3*k+1)**2+alp0(3*k+2)**2)
          xnorm(k) = fact
          xn(k) = alp0(3*k+1)*fact
          yn(k) = alp0(3*k+2)*fact
          zn(k) = fact
        enddo

        ps(1) = 0.d0
        ps(2) = 0.d0
        ps(3) = 0.d0
        do  k = 1, n
          nn = 3*k+3
          ps(1) = ps(1) + xn(k)*alp0(nn)
          ps(2) = ps(2) + yn(k)*alp0(nn)
          ps(3) = ps(3) + zn(k)*alp0(nn)
        enddo
        ps(4) = sqrt(ps(1)**2+ps(2)**2+ps(3)**2)
        fact = 1./ps(4)
        ps(1) = ps(1)*fact
        ps(2) = ps(2)*fact
        ps(3) = ps(3)*fact

        bet0(1) = betc(1)
        bet0(2) = alp0(1)
        bet0(3) = alp0(2)
        bet0(4) = alp0(3)
        bet0(5) = ps(1) / ps(3)
        bet0(6) = ps(2) / ps(3)
        bet0(7) = ps(4)

      endif

c---> fill Jacobian for secondary vertex

      do  i = 1, n3
        DbDa(1,i) = 0.d0
      enddo
      do  i = 2, 4
        do  j = 1, 3
          DbDa(i,j) = 0.d0
        enddo
        DbDa(i,i-1) = 1.d0
      enddo
      do  k = 1, n
        nn = 3*k+1
        do  i = 2, 4
          DbDa(i,nn)   = 0.d0
          DbDa(i,nn+1) = 0.d0
          DbDa(i,nn+2) = 0.d0
        enddo
      enddo
      do  i = 5, 7
        do  j = 1, 3
          DbDa(i,j) = 0.d0
        enddo
      enddo
      do  k = 1, n
        nn = 3*k+1
        fact = alp0(nn+2)*zn(k)/(ps(4)*ps(3))
        DbDa(5,nn) = fact*(1.d0-xn(k)**2+ps(1)/ps(3)*xn(k)*zn(k))
        DbDa(6,nn) = fact*xn(k)*(ps(2)/ps(3)*zn(k)-yn(k))
        nn = nn+1
        DbDa(5,nn) = fact*yn(k)*(ps(1)/ps(3)*zn(k)-xn(k))
        DbDa(6,nn) = fact*(1.d0-yn(k)**2+ps(2)/ps(3)*yn(k)*zn(k))
        nn = nn-1
        fact = alp0(nn+2)*zn(k)
        DbDa(7,nn) = fact*(ps(1)*(1.d0-xn(k)**2)-ps(2)*xn(k)*yn(k)
     1                                          -ps(3)*xn(k)*zn(k))
        nn = nn+1
        DbDa(7,nn) = fact*(ps(2)*(1.d0-yn(k)**2)-ps(1)*xn(k)*yn(k)
     1                                          -ps(3)*yn(k)*zn(k))
        nn = nn+1
        fact = 1.d0/(ps(4)*ps(3))
        DbDa(5,nn) = fact*(xn(k)-ps(1)/ps(3)*zn(k))
        DbDa(6,nn) = fact*(yn(k)-ps(2)/ps(3)*zn(k))
        DbDa(7,nn) = ps(1)*xn(k) + ps(2)*yn(k) + ps(3)*zn(k)
      enddo

c---> fill transported covariance matrix for secondary vertex

      do  i = 2, 7
        nn = i-1
        do  j = i, 7
          mm = j-1
          Vbet(nn,mm) = 0.d0
          do  k = 1, n3
            sum = 0.d0
            do  l = 1, n3
              sum = sum + Valp(k,l)*DbDa(j,l)
            enddo
            Vbet(nn,mm) = Vbet(nn,mm) + DbDa(i,k)*sum
          enddo
          Vbet(mm,nn) = Vbet(nn,mm)
        enddo
      enddo

      call DSINV(6,Vbet,6,ifail)
      if (ifail.eq.-1)   goto 998


c---> fill Jacobian for primary vertex

      M(1,1) = ps(1)
      M(2,1) = ps(2)
      M(3,1) = ps(3)
      do  i = 1, 3
        do  j = 2, 4
          M(i,j) = 0.d0
        enddo
      enddo
      M(1,2) = -1.d0
      M(2,3) = -1.d0
      M(3,4) = -1.d0
      M(1,7) = 0.d0
      M(2,7) = 0.d0
      M(3,7) = 0.d0
      fact = bet0(1)*ps(3)
      M(1,5) = fact*(1.d0-ps(1)**2)
      M(2,5) = -fact*ps(1)*ps(2)
      M(3,5) = -fact*ps(1)*ps(3)
      M(1,6) = M(2,5)
      M(2,6) = fact*(1.d0-ps(2)**2)
      M(3,6) = -fact*ps(2)*ps(3)

c---> fill auxilliary matrices

      do  i = 1, 3
        do  j = 1, 7
          CM(i,j) = 0.d0
          do  k = 1, 3
            CM(i,j) = CM(i,j) + Cp(i,k)*M(k,j)
          enddo
        enddo
      enddo
      do  i = 1, 7
        do  j = i, 7
          MCM(i,j) = 0.d0
          do  k = 1, 3
            MCM(i,j) = MCM(i,j) + M(k,i)*CM(k,j)
          enddo
          MCM(j,i) = MCM(i,j)
        enddo
      enddo
      do i = 1, 7
        Vbetc(i,1) = MCM(i,1)
        Vbetc(1,i) = MCM(1,i)
      enddo
      do  i = 2, 7
        nn = i-1
        do  j = i, 7
          Vbetc(i,j) = MCM(i,j)+Vbet(nn,j-1)
          Vbetc(j,i) = Vbetc(i,j)
        enddo
      enddo
      call DSINV(7,Vbetc,7,ifail)
      if ( ifail.eq.-1 ) goto 998

c--->  get new fit parameters

      do  i = 1, 3
        Mdb(i) = 0.d0
        do  j = 1, 7
          Mdb(i) = Mdb(i) + M(i,j)*(bet0(j)-bet(j))
        enddo
      enddo

      do  i = 1, 3
        delt0(i) = parp(i)+bet0(1)*ps(i)-bet0(i+1)
      enddo

      do  i = 1, 7
        betc(i) = bet(i)
        do  j = 1, 7
          sum = 0.d0
          do  k = 1, 3
            sum = sum + CM(k,j)*(Mdb(k)-delt0(k))
          enddo
          betc(i) = betc(i) + Vbetc(i,j)*sum
        enddo
      enddo

c--->   get chi**2

      do  i = 1, 7
        delbet(i) = betc(i) - bet(i)
      enddo
      do  i = 1, 3
        dMdb(i) = delt0(i)
        do  k = 1, 7
          dMdb(i) = dMdb(i) + M(i,k)*(betc(k)-bet0(k))
        enddo
      enddo
      chi2c = 0.d0
      do  k = 1, 6
        sum = 0.
        do  l = 1, 6
          sum = sum + Vbet(k,l)*delbet(l+1)
        enddo
        chi2c = chi2c + delbet(k+1) * sum
      enddo
      do  k = 1, 3
        sum = 0.
        do  l = 1, 3
          sum = sum + Cp(k,l)*dMdb(l)
        enddo
        chi2c = chi2c + dMdb(k) * sum
      enddo

      if ( abs(chi2c-ch2old) .gt. 0.00001 ) then
        ch2old = chi2c
        do  i = 1, 6
          Adab(i) = bet0(i+1) - betc(i+1)
          do  j = 1, n3
            Adab(i) = Adab(i) + DbDa(i+1,j)*(pars(j)-alp0(j))
          enddo
        enddo
        do  i = 1, 6
          VAdab(i) = 0.d0
          do  j = 1, 6
            VAdab(i) = VAdab(i) + Vbet(i,j)*Adab(j)
          enddo
        enddo
        do  i = 1, n3
          sum = 0.d0
          do  j = 1, 6
            sum = sum + DbDa(j+1,i)*VAdab(j)
          enddo
          AVAdab(i) = sum
        enddo
        do  i = 1, n3
          sum = 0.d0
          do  j = 1, n3
            sum = sum + Valp(i,j)*AVAdab(j)
          enddo
          alp0(i) = pars(i)-sum
        enddo
        goto 100
      endif

c---> set output parameters
      do  i = 1, 7
        par(i) = betc(i)
        do  j = 1, 7
          cov(i,j) = Vbetc(i,j)
        enddo
      enddo

      flag = 0

      return

  998 continue
      flag = -1

  999 continue

      return
      end
